import typing, clr, abc
from System.Reflection import Assembly, CustomAttributeData, TypeInfo, MethodInfo, Module, ManifestResourceInfo, AssemblyName, ConstructorInfo, MethodAttributes, CallingConventions, MemberTypes, MethodImplAttributes, ParameterAttributes, FieldInfo, PropertyInfo, ParameterInfo, ICustomAttributeProvider, BindingFlags, Binder, TypeAttributes, EventInfo, MemberInfo, MethodBase, GenericParameterAttributes, FieldAttributes, LocalVariableInfo, PropertyAttributes, EventAttributes
from System.Collections.Generic import IEnumerable_1
from System.Security import SecurityRuleSet
from System import Array_1, RuntimeMethodHandle, RuntimeFieldHandle, RuntimeTypeHandle, Delegate, Guid, IEquatable_1, ModuleHandle
from System.IO import FileStream, Stream
from System.Globalization import CultureInfo
from System.Runtime.InteropServices import StructLayoutAttribute, CallingConvention, CharSet

class AssemblyBuilder(Assembly):
    @property
    def CodeBase(self) -> str: ...
    @property
    def CustomAttributes(self) -> IEnumerable_1[CustomAttributeData]: ...
    @property
    def DefinedTypes(self) -> IEnumerable_1[TypeInfo]: ...
    @property
    def EntryPoint(self) -> MethodInfo: ...
    @property
    def EscapedCodeBase(self) -> str: ...
    @property
    def ExportedTypes(self) -> IEnumerable_1[typing.Type[typing.Any]]: ...
    @property
    def FullName(self) -> str: ...
    @property
    def GlobalAssemblyCache(self) -> bool: ...
    @property
    def HostContext(self) -> int: ...
    @property
    def ImageRuntimeVersion(self) -> str: ...
    @property
    def IsCollectible(self) -> bool: ...
    @property
    def IsDynamic(self) -> bool: ...
    @property
    def IsFullyTrusted(self) -> bool: ...
    @property
    def Location(self) -> str: ...
    @property
    def ManifestModule(self) -> Module: ...
    @property
    def Modules(self) -> IEnumerable_1[Module]: ...
    @property
    def ReflectionOnly(self) -> bool: ...
    @property
    def SecurityRuleSet(self) -> SecurityRuleSet: ...
    def DefineDynamicModule(self, name: str) -> ModuleBuilder: ...
    def GetDynamicModule(self, name: str) -> ModuleBuilder: ...
    def GetExportedTypes(self) -> Array_1[typing.Type[typing.Any]]: ...
    def GetFile(self, name: str) -> FileStream: ...
    def GetFiles(self, getResourceModules: bool) -> Array_1[FileStream]: ...
    def GetManifestResourceInfo(self, resourceName: str) -> ManifestResourceInfo: ...
    def GetManifestResourceNames(self) -> Array_1[str]: ...
    # Skipped DefineDynamicAssembly due to it being static, abstract and generic.

    DefineDynamicAssembly : DefineDynamicAssembly_MethodGroup
    class DefineDynamicAssembly_MethodGroup:
        @typing.overload
        def __call__(self, name: AssemblyName, access: AssemblyBuilderAccess) -> AssemblyBuilder:...
        @typing.overload
        def __call__(self, name: AssemblyName, access: AssemblyBuilderAccess, assemblyAttributes: IEnumerable_1[CustomAttributeBuilder]) -> AssemblyBuilder:...

    # Skipped GetManifestResourceStream due to it being static, abstract and generic.

    GetManifestResourceStream : GetManifestResourceStream_MethodGroup
    class GetManifestResourceStream_MethodGroup:
        @typing.overload
        def __call__(self, name: str) -> Stream:...
        @typing.overload
        def __call__(self, type: typing.Type[typing.Any], name: str) -> Stream:...

    # Skipped SetCustomAttribute due to it being static, abstract and generic.

    SetCustomAttribute : SetCustomAttribute_MethodGroup
    class SetCustomAttribute_MethodGroup:
        @typing.overload
        def __call__(self, customBuilder: CustomAttributeBuilder) -> None:...
        @typing.overload
        def __call__(self, con: ConstructorInfo, binaryAttribute: Array_1[int]) -> None:...



class AssemblyBuilderAccess(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    Run : AssemblyBuilderAccess # 1
    RunAndCollect : AssemblyBuilderAccess # 9


class ConstructorBuilder(ConstructorInfo):
    @property
    def Attributes(self) -> MethodAttributes: ...
    @property
    def CallingConvention(self) -> CallingConventions: ...
    @property
    def ContainsGenericParameters(self) -> bool: ...
    @property
    def CustomAttributes(self) -> IEnumerable_1[CustomAttributeData]: ...
    @property
    def DeclaringType(self) -> typing.Type[typing.Any]: ...
    @property
    def InitLocals(self) -> bool: ...
    @InitLocals.setter
    def InitLocals(self, value: bool) -> bool: ...
    @property
    def IsAbstract(self) -> bool: ...
    @property
    def IsAssembly(self) -> bool: ...
    @property
    def IsCollectible(self) -> bool: ...
    @property
    def IsConstructedGenericMethod(self) -> bool: ...
    @property
    def IsConstructor(self) -> bool: ...
    @property
    def IsFamily(self) -> bool: ...
    @property
    def IsFamilyAndAssembly(self) -> bool: ...
    @property
    def IsFamilyOrAssembly(self) -> bool: ...
    @property
    def IsFinal(self) -> bool: ...
    @property
    def IsGenericMethod(self) -> bool: ...
    @property
    def IsGenericMethodDefinition(self) -> bool: ...
    @property
    def IsHideBySig(self) -> bool: ...
    @property
    def IsPrivate(self) -> bool: ...
    @property
    def IsPublic(self) -> bool: ...
    @property
    def IsSecurityCritical(self) -> bool: ...
    @property
    def IsSecuritySafeCritical(self) -> bool: ...
    @property
    def IsSecurityTransparent(self) -> bool: ...
    @property
    def IsSpecialName(self) -> bool: ...
    @property
    def IsStatic(self) -> bool: ...
    @property
    def IsVirtual(self) -> bool: ...
    @property
    def MemberType(self) -> MemberTypes: ...
    @property
    def MetadataToken(self) -> int: ...
    @property
    def MethodHandle(self) -> RuntimeMethodHandle: ...
    @property
    def MethodImplementationFlags(self) -> MethodImplAttributes: ...
    @property
    def Module(self) -> Module: ...
    @property
    def Name(self) -> str: ...
    @property
    def ReflectedType(self) -> typing.Type[typing.Any]: ...
    def DefineParameter(self, iSequence: int, attributes: ParameterAttributes, strParamName: str) -> ParameterBuilder: ...
    def SetImplementationFlags(self, attributes: MethodImplAttributes) -> None: ...
    # Skipped GetILGenerator due to it being static, abstract and generic.

    GetILGenerator : GetILGenerator_MethodGroup
    class GetILGenerator_MethodGroup:
        @typing.overload
        def __call__(self) -> ILGenerator:...
        @typing.overload
        def __call__(self, streamSize: int) -> ILGenerator:...

    # Skipped SetCustomAttribute due to it being static, abstract and generic.

    SetCustomAttribute : SetCustomAttribute_MethodGroup
    class SetCustomAttribute_MethodGroup:
        @typing.overload
        def __call__(self, customBuilder: CustomAttributeBuilder) -> None:...
        @typing.overload
        def __call__(self, con: ConstructorInfo, binaryAttribute: Array_1[int]) -> None:...



class CustomAttributeBuilder:
    @typing.overload
    def __init__(self, con: ConstructorInfo, constructorArgs: Array_1[typing.Any]) -> None: ...
    @typing.overload
    def __init__(self, con: ConstructorInfo, constructorArgs: Array_1[typing.Any], namedFields: Array_1[FieldInfo], fieldValues: Array_1[typing.Any]) -> None: ...
    @typing.overload
    def __init__(self, con: ConstructorInfo, constructorArgs: Array_1[typing.Any], namedProperties: Array_1[PropertyInfo], propertyValues: Array_1[typing.Any]) -> None: ...
    @typing.overload
    def __init__(self, con: ConstructorInfo, constructorArgs: Array_1[typing.Any], namedProperties: Array_1[PropertyInfo], propertyValues: Array_1[typing.Any], namedFields: Array_1[FieldInfo], fieldValues: Array_1[typing.Any]) -> None: ...


class DynamicILInfo:
    @property
    def DynamicMethod(self) -> DynamicMethod: ...
    # Skipped GetTokenFor due to it being static, abstract and generic.

    GetTokenFor : GetTokenFor_MethodGroup
    class GetTokenFor_MethodGroup:
        @typing.overload
        def __call__(self, signature: Array_1[int]) -> int:...
        @typing.overload
        def __call__(self, method: DynamicMethod) -> int:...
        @typing.overload
        def __call__(self, field: RuntimeFieldHandle) -> int:...
        @typing.overload
        def __call__(self, literal: str) -> int:...
        @typing.overload
        def __call__(self, method: RuntimeMethodHandle) -> int:...
        @typing.overload
        def __call__(self, type: RuntimeTypeHandle) -> int:...
        @typing.overload
        def __call__(self, field: RuntimeFieldHandle, contextType: RuntimeTypeHandle) -> int:...
        @typing.overload
        def __call__(self, method: RuntimeMethodHandle, contextType: RuntimeTypeHandle) -> int:...

    # Skipped SetCode due to it being static, abstract and generic.

    SetCode : SetCode_MethodGroup
    class SetCode_MethodGroup:
        @typing.overload
        def __call__(self, code: Array_1[int], maxStackSize: int) -> None:...
        @typing.overload
        def __call__(self, code: clr.Reference[int], codeSize: int, maxStackSize: int) -> None:...

    # Skipped SetExceptions due to it being static, abstract and generic.

    SetExceptions : SetExceptions_MethodGroup
    class SetExceptions_MethodGroup:
        @typing.overload
        def __call__(self, exceptions: Array_1[int]) -> None:...
        @typing.overload
        def __call__(self, exceptions: clr.Reference[int], exceptionsSize: int) -> None:...

    # Skipped SetLocalSignature due to it being static, abstract and generic.

    SetLocalSignature : SetLocalSignature_MethodGroup
    class SetLocalSignature_MethodGroup:
        @typing.overload
        def __call__(self, localSignature: Array_1[int]) -> None:...
        @typing.overload
        def __call__(self, localSignature: clr.Reference[int], signatureSize: int) -> None:...



class DynamicMethod(MethodInfo):
    @typing.overload
    def __init__(self, name: str, attributes: MethodAttributes, callingConvention: CallingConventions, returnType: typing.Type[typing.Any], parameterTypes: Array_1[typing.Type[typing.Any]], m: Module, skipVisibility: bool) -> None: ...
    @typing.overload
    def __init__(self, name: str, attributes: MethodAttributes, callingConvention: CallingConventions, returnType: typing.Type[typing.Any], parameterTypes: Array_1[typing.Type[typing.Any]], owner: typing.Type[typing.Any], skipVisibility: bool) -> None: ...
    @typing.overload
    def __init__(self, name: str, returnType: typing.Type[typing.Any], parameterTypes: Array_1[typing.Type[typing.Any]]) -> None: ...
    @typing.overload
    def __init__(self, name: str, returnType: typing.Type[typing.Any], parameterTypes: Array_1[typing.Type[typing.Any]], m: Module) -> None: ...
    @typing.overload
    def __init__(self, name: str, returnType: typing.Type[typing.Any], parameterTypes: Array_1[typing.Type[typing.Any]], m: Module, skipVisibility: bool) -> None: ...
    @typing.overload
    def __init__(self, name: str, returnType: typing.Type[typing.Any], parameterTypes: Array_1[typing.Type[typing.Any]], owner: typing.Type[typing.Any]) -> None: ...
    @typing.overload
    def __init__(self, name: str, returnType: typing.Type[typing.Any], parameterTypes: Array_1[typing.Type[typing.Any]], owner: typing.Type[typing.Any], skipVisibility: bool) -> None: ...
    @typing.overload
    def __init__(self, name: str, returnType: typing.Type[typing.Any], parameterTypes: Array_1[typing.Type[typing.Any]], restrictedSkipVisibility: bool) -> None: ...
    @property
    def Attributes(self) -> MethodAttributes: ...
    @property
    def CallingConvention(self) -> CallingConventions: ...
    @property
    def ContainsGenericParameters(self) -> bool: ...
    @property
    def CustomAttributes(self) -> IEnumerable_1[CustomAttributeData]: ...
    @property
    def DeclaringType(self) -> typing.Type[typing.Any]: ...
    @property
    def InitLocals(self) -> bool: ...
    @InitLocals.setter
    def InitLocals(self, value: bool) -> bool: ...
    @property
    def IsAbstract(self) -> bool: ...
    @property
    def IsAssembly(self) -> bool: ...
    @property
    def IsCollectible(self) -> bool: ...
    @property
    def IsConstructedGenericMethod(self) -> bool: ...
    @property
    def IsConstructor(self) -> bool: ...
    @property
    def IsFamily(self) -> bool: ...
    @property
    def IsFamilyAndAssembly(self) -> bool: ...
    @property
    def IsFamilyOrAssembly(self) -> bool: ...
    @property
    def IsFinal(self) -> bool: ...
    @property
    def IsGenericMethod(self) -> bool: ...
    @property
    def IsGenericMethodDefinition(self) -> bool: ...
    @property
    def IsHideBySig(self) -> bool: ...
    @property
    def IsPrivate(self) -> bool: ...
    @property
    def IsPublic(self) -> bool: ...
    @property
    def IsSecurityCritical(self) -> bool: ...
    @property
    def IsSecuritySafeCritical(self) -> bool: ...
    @property
    def IsSecurityTransparent(self) -> bool: ...
    @property
    def IsSpecialName(self) -> bool: ...
    @property
    def IsStatic(self) -> bool: ...
    @property
    def IsVirtual(self) -> bool: ...
    @property
    def MemberType(self) -> MemberTypes: ...
    @property
    def MetadataToken(self) -> int: ...
    @property
    def MethodHandle(self) -> RuntimeMethodHandle: ...
    @property
    def MethodImplementationFlags(self) -> MethodImplAttributes: ...
    @property
    def Module(self) -> Module: ...
    @property
    def Name(self) -> str: ...
    @property
    def ReflectedType(self) -> typing.Type[typing.Any]: ...
    @property
    def ReturnParameter(self) -> ParameterInfo: ...
    @property
    def ReturnType(self) -> typing.Type[typing.Any]: ...
    @property
    def ReturnTypeCustomAttributes(self) -> ICustomAttributeProvider: ...
    def DefineParameter(self, position: int, attributes: ParameterAttributes, parameterName: str) -> ParameterBuilder: ...
    def GetBaseDefinition(self) -> MethodInfo: ...
    def GetDynamicILInfo(self) -> DynamicILInfo: ...
    def GetMethodImplementationFlags(self) -> MethodImplAttributes: ...
    def GetParameters(self) -> Array_1[ParameterInfo]: ...
    def Invoke(self, obj: typing.Any, invokeAttr: BindingFlags, binder: Binder, parameters: Array_1[typing.Any], culture: CultureInfo) -> typing.Any: ...
    def IsDefined(self, attributeType: typing.Type[typing.Any], inherit: bool) -> bool: ...
    def ToString(self) -> str: ...
    # Skipped CreateDelegate due to it being static, abstract and generic.

    CreateDelegate : CreateDelegate_MethodGroup
    class CreateDelegate_MethodGroup:
        @typing.overload
        def __call__(self, delegateType: typing.Type[typing.Any]) -> Delegate:...
        @typing.overload
        def __call__(self, delegateType: typing.Type[typing.Any], target: typing.Any) -> Delegate:...

    # Skipped GetCustomAttributes due to it being static, abstract and generic.

    GetCustomAttributes : GetCustomAttributes_MethodGroup
    class GetCustomAttributes_MethodGroup:
        @typing.overload
        def __call__(self, inherit: bool) -> Array_1[typing.Any]:...
        @typing.overload
        def __call__(self, attributeType: typing.Type[typing.Any], inherit: bool) -> Array_1[typing.Any]:...

    # Skipped GetILGenerator due to it being static, abstract and generic.

    GetILGenerator : GetILGenerator_MethodGroup
    class GetILGenerator_MethodGroup:
        @typing.overload
        def __call__(self) -> ILGenerator:...
        @typing.overload
        def __call__(self, streamSize: int) -> ILGenerator:...



class EnumBuilder(TypeInfo):
    @property
    def Assembly(self) -> Assembly: ...
    @property
    def AssemblyQualifiedName(self) -> str: ...
    @property
    def Attributes(self) -> TypeAttributes: ...
    @property
    def BaseType(self) -> typing.Type[typing.Any]: ...
    @property
    def ContainsGenericParameters(self) -> bool: ...
    @property
    def CustomAttributes(self) -> IEnumerable_1[CustomAttributeData]: ...
    @property
    def DeclaredConstructors(self) -> IEnumerable_1[ConstructorInfo]: ...
    @property
    def DeclaredEvents(self) -> IEnumerable_1[EventInfo]: ...
    @property
    def DeclaredFields(self) -> IEnumerable_1[FieldInfo]: ...
    @property
    def DeclaredMembers(self) -> IEnumerable_1[MemberInfo]: ...
    @property
    def DeclaredMethods(self) -> IEnumerable_1[MethodInfo]: ...
    @property
    def DeclaredNestedTypes(self) -> IEnumerable_1[TypeInfo]: ...
    @property
    def DeclaredProperties(self) -> IEnumerable_1[PropertyInfo]: ...
    @property
    def DeclaringMethod(self) -> MethodBase: ...
    @property
    def DeclaringType(self) -> typing.Type[typing.Any]: ...
    @property
    def FullName(self) -> str: ...
    @property
    def GenericParameterAttributes(self) -> GenericParameterAttributes: ...
    @property
    def GenericParameterPosition(self) -> int: ...
    @property
    def GenericTypeArguments(self) -> Array_1[typing.Type[typing.Any]]: ...
    @property
    def GenericTypeParameters(self) -> Array_1[typing.Type[typing.Any]]: ...
    @property
    def GUID(self) -> Guid: ...
    @property
    def HasElementType(self) -> bool: ...
    @property
    def ImplementedInterfaces(self) -> IEnumerable_1[typing.Type[typing.Any]]: ...
    @property
    def IsAbstract(self) -> bool: ...
    @property
    def IsAnsiClass(self) -> bool: ...
    @property
    def IsArray(self) -> bool: ...
    @property
    def IsAutoClass(self) -> bool: ...
    @property
    def IsAutoLayout(self) -> bool: ...
    @property
    def IsByRef(self) -> bool: ...
    @property
    def IsByRefLike(self) -> bool: ...
    @property
    def IsClass(self) -> bool: ...
    @property
    def IsCollectible(self) -> bool: ...
    @property
    def IsCOMObject(self) -> bool: ...
    @property
    def IsConstructedGenericType(self) -> bool: ...
    @property
    def IsContextful(self) -> bool: ...
    @property
    def IsEnum(self) -> bool: ...
    @property
    def IsExplicitLayout(self) -> bool: ...
    @property
    def IsFunctionPointer(self) -> bool: ...
    @property
    def IsGenericMethodParameter(self) -> bool: ...
    @property
    def IsGenericParameter(self) -> bool: ...
    @property
    def IsGenericType(self) -> bool: ...
    @property
    def IsGenericTypeDefinition(self) -> bool: ...
    @property
    def IsGenericTypeParameter(self) -> bool: ...
    @property
    def IsImport(self) -> bool: ...
    @property
    def IsInterface(self) -> bool: ...
    @property
    def IsLayoutSequential(self) -> bool: ...
    @property
    def IsMarshalByRef(self) -> bool: ...
    @property
    def IsNested(self) -> bool: ...
    @property
    def IsNestedAssembly(self) -> bool: ...
    @property
    def IsNestedFamANDAssem(self) -> bool: ...
    @property
    def IsNestedFamily(self) -> bool: ...
    @property
    def IsNestedFamORAssem(self) -> bool: ...
    @property
    def IsNestedPrivate(self) -> bool: ...
    @property
    def IsNestedPublic(self) -> bool: ...
    @property
    def IsNotPublic(self) -> bool: ...
    @property
    def IsPointer(self) -> bool: ...
    @property
    def IsPrimitive(self) -> bool: ...
    @property
    def IsPublic(self) -> bool: ...
    @property
    def IsSealed(self) -> bool: ...
    @property
    def IsSecurityCritical(self) -> bool: ...
    @property
    def IsSecuritySafeCritical(self) -> bool: ...
    @property
    def IsSecurityTransparent(self) -> bool: ...
    @property
    def IsSerializable(self) -> bool: ...
    @property
    def IsSignatureType(self) -> bool: ...
    @property
    def IsSpecialName(self) -> bool: ...
    @property
    def IsSZArray(self) -> bool: ...
    @property
    def IsTypeDefinition(self) -> bool: ...
    @property
    def IsUnicodeClass(self) -> bool: ...
    @property
    def IsUnmanagedFunctionPointer(self) -> bool: ...
    @property
    def IsValueType(self) -> bool: ...
    @property
    def IsVariableBoundArray(self) -> bool: ...
    @property
    def IsVisible(self) -> bool: ...
    @property
    def MemberType(self) -> MemberTypes: ...
    @property
    def MetadataToken(self) -> int: ...
    @property
    def Module(self) -> Module: ...
    @property
    def Name(self) -> str: ...
    @property
    def Namespace(self) -> str: ...
    @property
    def ReflectedType(self) -> typing.Type[typing.Any]: ...
    @property
    def StructLayoutAttribute(self) -> StructLayoutAttribute: ...
    @property
    def TypeHandle(self) -> RuntimeTypeHandle: ...
    @property
    def TypeInitializer(self) -> ConstructorInfo: ...
    @property
    def UnderlyingField(self) -> FieldBuilder: ...
    @property
    def UnderlyingSystemType(self) -> typing.Type[typing.Any]: ...
    def CreateType(self) -> typing.Type[typing.Any]: ...
    def CreateTypeInfo(self) -> TypeInfo: ...
    def DefineLiteral(self, literalName: str, literalValue: typing.Any) -> FieldBuilder: ...
    def MakeByRefType(self) -> typing.Type[typing.Any]: ...
    def MakePointerType(self) -> typing.Type[typing.Any]: ...
    # Skipped MakeArrayType due to it being static, abstract and generic.

    MakeArrayType : MakeArrayType_MethodGroup
    class MakeArrayType_MethodGroup:
        @typing.overload
        def __call__(self) -> typing.Type[typing.Any]:...
        @typing.overload
        def __call__(self, rank: int) -> typing.Type[typing.Any]:...

    # Skipped SetCustomAttribute due to it being static, abstract and generic.

    SetCustomAttribute : SetCustomAttribute_MethodGroup
    class SetCustomAttribute_MethodGroup:
        @typing.overload
        def __call__(self, customBuilder: CustomAttributeBuilder) -> None:...
        @typing.overload
        def __call__(self, con: ConstructorInfo, binaryAttribute: Array_1[int]) -> None:...



class EventBuilder(abc.ABC):
    def AddOtherMethod(self, mdBuilder: MethodBuilder) -> None: ...
    def SetAddOnMethod(self, mdBuilder: MethodBuilder) -> None: ...
    def SetRaiseMethod(self, mdBuilder: MethodBuilder) -> None: ...
    def SetRemoveOnMethod(self, mdBuilder: MethodBuilder) -> None: ...
    # Skipped SetCustomAttribute due to it being static, abstract and generic.

    SetCustomAttribute : SetCustomAttribute_MethodGroup
    class SetCustomAttribute_MethodGroup:
        @typing.overload
        def __call__(self, customBuilder: CustomAttributeBuilder) -> None:...
        @typing.overload
        def __call__(self, con: ConstructorInfo, binaryAttribute: Array_1[int]) -> None:...



class FieldBuilder(FieldInfo):
    @property
    def Attributes(self) -> FieldAttributes: ...
    @property
    def CustomAttributes(self) -> IEnumerable_1[CustomAttributeData]: ...
    @property
    def DeclaringType(self) -> typing.Type[typing.Any]: ...
    @property
    def FieldHandle(self) -> RuntimeFieldHandle: ...
    @property
    def FieldType(self) -> typing.Type[typing.Any]: ...
    @property
    def IsAssembly(self) -> bool: ...
    @property
    def IsCollectible(self) -> bool: ...
    @property
    def IsFamily(self) -> bool: ...
    @property
    def IsFamilyAndAssembly(self) -> bool: ...
    @property
    def IsFamilyOrAssembly(self) -> bool: ...
    @property
    def IsInitOnly(self) -> bool: ...
    @property
    def IsLiteral(self) -> bool: ...
    @property
    def IsNotSerialized(self) -> bool: ...
    @property
    def IsPinvokeImpl(self) -> bool: ...
    @property
    def IsPrivate(self) -> bool: ...
    @property
    def IsPublic(self) -> bool: ...
    @property
    def IsSecurityCritical(self) -> bool: ...
    @property
    def IsSecuritySafeCritical(self) -> bool: ...
    @property
    def IsSecurityTransparent(self) -> bool: ...
    @property
    def IsSpecialName(self) -> bool: ...
    @property
    def IsStatic(self) -> bool: ...
    @property
    def MemberType(self) -> MemberTypes: ...
    @property
    def MetadataToken(self) -> int: ...
    @property
    def Module(self) -> Module: ...
    @property
    def Name(self) -> str: ...
    @property
    def ReflectedType(self) -> typing.Type[typing.Any]: ...
    def SetConstant(self, defaultValue: typing.Any) -> None: ...
    def SetOffset(self, iOffset: int) -> None: ...
    # Skipped SetCustomAttribute due to it being static, abstract and generic.

    SetCustomAttribute : SetCustomAttribute_MethodGroup
    class SetCustomAttribute_MethodGroup:
        @typing.overload
        def __call__(self, customBuilder: CustomAttributeBuilder) -> None:...
        @typing.overload
        def __call__(self, con: ConstructorInfo, binaryAttribute: Array_1[int]) -> None:...



class FlowControl(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    Branch : FlowControl # 0
    Break : FlowControl # 1
    Call : FlowControl # 2
    Cond_Branch : FlowControl # 3
    Meta : FlowControl # 4
    Next : FlowControl # 5
    Phi : FlowControl # 6
    Return : FlowControl # 7
    Throw : FlowControl # 8


class GenericTypeParameterBuilder(TypeInfo):
    @property
    def Assembly(self) -> Assembly: ...
    @property
    def AssemblyQualifiedName(self) -> str: ...
    @property
    def Attributes(self) -> TypeAttributes: ...
    @property
    def BaseType(self) -> typing.Type[typing.Any]: ...
    @property
    def ContainsGenericParameters(self) -> bool: ...
    @property
    def CustomAttributes(self) -> IEnumerable_1[CustomAttributeData]: ...
    @property
    def DeclaredConstructors(self) -> IEnumerable_1[ConstructorInfo]: ...
    @property
    def DeclaredEvents(self) -> IEnumerable_1[EventInfo]: ...
    @property
    def DeclaredFields(self) -> IEnumerable_1[FieldInfo]: ...
    @property
    def DeclaredMembers(self) -> IEnumerable_1[MemberInfo]: ...
    @property
    def DeclaredMethods(self) -> IEnumerable_1[MethodInfo]: ...
    @property
    def DeclaredNestedTypes(self) -> IEnumerable_1[TypeInfo]: ...
    @property
    def DeclaredProperties(self) -> IEnumerable_1[PropertyInfo]: ...
    @property
    def DeclaringMethod(self) -> MethodBase: ...
    @property
    def DeclaringType(self) -> typing.Type[typing.Any]: ...
    @property
    def FullName(self) -> str: ...
    @property
    def GenericParameterAttributes(self) -> GenericParameterAttributes: ...
    @property
    def GenericParameterPosition(self) -> int: ...
    @property
    def GenericTypeArguments(self) -> Array_1[typing.Type[typing.Any]]: ...
    @property
    def GenericTypeParameters(self) -> Array_1[typing.Type[typing.Any]]: ...
    @property
    def GUID(self) -> Guid: ...
    @property
    def HasElementType(self) -> bool: ...
    @property
    def ImplementedInterfaces(self) -> IEnumerable_1[typing.Type[typing.Any]]: ...
    @property
    def IsAbstract(self) -> bool: ...
    @property
    def IsAnsiClass(self) -> bool: ...
    @property
    def IsArray(self) -> bool: ...
    @property
    def IsAutoClass(self) -> bool: ...
    @property
    def IsAutoLayout(self) -> bool: ...
    @property
    def IsByRef(self) -> bool: ...
    @property
    def IsByRefLike(self) -> bool: ...
    @property
    def IsClass(self) -> bool: ...
    @property
    def IsCollectible(self) -> bool: ...
    @property
    def IsCOMObject(self) -> bool: ...
    @property
    def IsConstructedGenericType(self) -> bool: ...
    @property
    def IsContextful(self) -> bool: ...
    @property
    def IsEnum(self) -> bool: ...
    @property
    def IsExplicitLayout(self) -> bool: ...
    @property
    def IsFunctionPointer(self) -> bool: ...
    @property
    def IsGenericMethodParameter(self) -> bool: ...
    @property
    def IsGenericParameter(self) -> bool: ...
    @property
    def IsGenericType(self) -> bool: ...
    @property
    def IsGenericTypeDefinition(self) -> bool: ...
    @property
    def IsGenericTypeParameter(self) -> bool: ...
    @property
    def IsImport(self) -> bool: ...
    @property
    def IsInterface(self) -> bool: ...
    @property
    def IsLayoutSequential(self) -> bool: ...
    @property
    def IsMarshalByRef(self) -> bool: ...
    @property
    def IsNested(self) -> bool: ...
    @property
    def IsNestedAssembly(self) -> bool: ...
    @property
    def IsNestedFamANDAssem(self) -> bool: ...
    @property
    def IsNestedFamily(self) -> bool: ...
    @property
    def IsNestedFamORAssem(self) -> bool: ...
    @property
    def IsNestedPrivate(self) -> bool: ...
    @property
    def IsNestedPublic(self) -> bool: ...
    @property
    def IsNotPublic(self) -> bool: ...
    @property
    def IsPointer(self) -> bool: ...
    @property
    def IsPrimitive(self) -> bool: ...
    @property
    def IsPublic(self) -> bool: ...
    @property
    def IsSealed(self) -> bool: ...
    @property
    def IsSecurityCritical(self) -> bool: ...
    @property
    def IsSecuritySafeCritical(self) -> bool: ...
    @property
    def IsSecurityTransparent(self) -> bool: ...
    @property
    def IsSerializable(self) -> bool: ...
    @property
    def IsSignatureType(self) -> bool: ...
    @property
    def IsSpecialName(self) -> bool: ...
    @property
    def IsSZArray(self) -> bool: ...
    @property
    def IsTypeDefinition(self) -> bool: ...
    @property
    def IsUnicodeClass(self) -> bool: ...
    @property
    def IsUnmanagedFunctionPointer(self) -> bool: ...
    @property
    def IsValueType(self) -> bool: ...
    @property
    def IsVariableBoundArray(self) -> bool: ...
    @property
    def IsVisible(self) -> bool: ...
    @property
    def MemberType(self) -> MemberTypes: ...
    @property
    def MetadataToken(self) -> int: ...
    @property
    def Module(self) -> Module: ...
    @property
    def Name(self) -> str: ...
    @property
    def Namespace(self) -> str: ...
    @property
    def ReflectedType(self) -> typing.Type[typing.Any]: ...
    @property
    def StructLayoutAttribute(self) -> StructLayoutAttribute: ...
    @property
    def TypeHandle(self) -> RuntimeTypeHandle: ...
    @property
    def TypeInitializer(self) -> ConstructorInfo: ...
    @property
    def UnderlyingSystemType(self) -> typing.Type[typing.Any]: ...
    def SetBaseTypeConstraint(self, baseTypeConstraint: typing.Type[typing.Any]) -> None: ...
    def SetGenericParameterAttributes(self, genericParameterAttributes: GenericParameterAttributes) -> None: ...
    def SetInterfaceConstraints(self, interfaceConstraints: Array_1[typing.Type[typing.Any]]) -> None: ...
    # Skipped SetCustomAttribute due to it being static, abstract and generic.

    SetCustomAttribute : SetCustomAttribute_MethodGroup
    class SetCustomAttribute_MethodGroup:
        @typing.overload
        def __call__(self, customBuilder: CustomAttributeBuilder) -> None:...
        @typing.overload
        def __call__(self, con: ConstructorInfo, binaryAttribute: Array_1[int]) -> None:...



class ILGenerator(abc.ABC):
    @property
    def ILOffset(self) -> int: ...
    @abc.abstractmethod
    def BeginCatchBlock(self, exceptionType: typing.Type[typing.Any]) -> None: ...
    @abc.abstractmethod
    def BeginExceptFilterBlock(self) -> None: ...
    @abc.abstractmethod
    def BeginExceptionBlock(self) -> Label: ...
    @abc.abstractmethod
    def BeginFaultBlock(self) -> None: ...
    @abc.abstractmethod
    def BeginFinallyBlock(self) -> None: ...
    @abc.abstractmethod
    def BeginScope(self) -> None: ...
    @abc.abstractmethod
    def DefineLabel(self) -> Label: ...
    @abc.abstractmethod
    def EmitCall(self, opcode: OpCode, methodInfo: MethodInfo, optionalParameterTypes: Array_1[typing.Type[typing.Any]]) -> None: ...
    @abc.abstractmethod
    def EndExceptionBlock(self) -> None: ...
    @abc.abstractmethod
    def EndScope(self) -> None: ...
    @abc.abstractmethod
    def MarkLabel(self, loc: Label) -> None: ...
    def ThrowException(self, excType: typing.Type[typing.Any]) -> None: ...
    @abc.abstractmethod
    def UsingNamespace(self, usingNamespace: str) -> None: ...
    # Skipped DeclareLocal due to it being static, abstract and generic.

    DeclareLocal : DeclareLocal_MethodGroup
    class DeclareLocal_MethodGroup:
        @typing.overload
        def __call__(self, localType: typing.Type[typing.Any]) -> LocalBuilder:...
        @typing.overload
        def __call__(self, localType: typing.Type[typing.Any], pinned: bool) -> LocalBuilder:...

    # Skipped Emit due to it being static, abstract and generic.

    Emit : Emit_MethodGroup
    class Emit_MethodGroup:
        @typing.overload
        def __call__(self, opcode: OpCode) -> None:...
        @typing.overload
        def __call__(self, opcode: OpCode, arg: float) -> None:...
        # Method Emit(opcode : OpCode, arg : Double) was skipped since it collides with above method
        # Method Emit(opcode : OpCode, arg : Byte) was skipped since it collides with above method
        # Method Emit(opcode : OpCode, arg : Int16) was skipped since it collides with above method
        # Method Emit(opcode : OpCode, arg : Int64) was skipped since it collides with above method
        # Method Emit(opcode : OpCode, arg : Int32) was skipped since it collides with above method
        # Method Emit(opcode : OpCode, arg : SByte) was skipped since it collides with above method
        @typing.overload
        def __call__(self, opcode: OpCode, labels: Array_1[Label]) -> None:...
        @typing.overload
        def __call__(self, opcode: OpCode, con: ConstructorInfo) -> None:...
        @typing.overload
        def __call__(self, opcode: OpCode, meth: MethodInfo) -> None:...
        @typing.overload
        def __call__(self, opcode: OpCode, cls: typing.Type[typing.Any]) -> None:...
        @typing.overload
        def __call__(self, opcode: OpCode, field: FieldInfo) -> None:...
        @typing.overload
        def __call__(self, opcode: OpCode, label: Label) -> None:...
        @typing.overload
        def __call__(self, opcode: OpCode, local: LocalBuilder) -> None:...
        @typing.overload
        def __call__(self, opcode: OpCode, str: str) -> None:...
        @typing.overload
        def __call__(self, opcode: OpCode, signature: SignatureHelper) -> None:...

    # Skipped EmitCalli due to it being static, abstract and generic.

    EmitCalli : EmitCalli_MethodGroup
    class EmitCalli_MethodGroup:
        @typing.overload
        def __call__(self, opcode: OpCode, unmanagedCallConv: CallingConvention, returnType: typing.Type[typing.Any], parameterTypes: Array_1[typing.Type[typing.Any]]) -> None:...
        @typing.overload
        def __call__(self, opcode: OpCode, callingConvention: CallingConventions, returnType: typing.Type[typing.Any], parameterTypes: Array_1[typing.Type[typing.Any]], optionalParameterTypes: Array_1[typing.Type[typing.Any]]) -> None:...

    # Skipped EmitWriteLine due to it being static, abstract and generic.

    EmitWriteLine : EmitWriteLine_MethodGroup
    class EmitWriteLine_MethodGroup:
        @typing.overload
        def __call__(self, fld: FieldInfo) -> None:...
        @typing.overload
        def __call__(self, localBuilder: LocalBuilder) -> None:...
        @typing.overload
        def __call__(self, value: str) -> None:...



class Label(IEquatable_1[Label]):
    def GetHashCode(self) -> int: ...
    def __eq__(self, a: Label, b: Label) -> bool: ...
    def __ne__(self, a: Label, b: Label) -> bool: ...
    # Skipped Equals due to it being static, abstract and generic.

    Equals : Equals_MethodGroup
    class Equals_MethodGroup:
        @typing.overload
        def __call__(self, obj: Label) -> bool:...
        @typing.overload
        def __call__(self, obj: typing.Any) -> bool:...



class LocalBuilder(LocalVariableInfo):
    @property
    def IsPinned(self) -> bool: ...
    @property
    def LocalIndex(self) -> int: ...
    @property
    def LocalType(self) -> typing.Type[typing.Any]: ...


class MethodBuilder(MethodInfo):
    @property
    def Attributes(self) -> MethodAttributes: ...
    @property
    def CallingConvention(self) -> CallingConventions: ...
    @property
    def ContainsGenericParameters(self) -> bool: ...
    @property
    def CustomAttributes(self) -> IEnumerable_1[CustomAttributeData]: ...
    @property
    def DeclaringType(self) -> typing.Type[typing.Any]: ...
    @property
    def InitLocals(self) -> bool: ...
    @InitLocals.setter
    def InitLocals(self, value: bool) -> bool: ...
    @property
    def IsAbstract(self) -> bool: ...
    @property
    def IsAssembly(self) -> bool: ...
    @property
    def IsCollectible(self) -> bool: ...
    @property
    def IsConstructedGenericMethod(self) -> bool: ...
    @property
    def IsConstructor(self) -> bool: ...
    @property
    def IsFamily(self) -> bool: ...
    @property
    def IsFamilyAndAssembly(self) -> bool: ...
    @property
    def IsFamilyOrAssembly(self) -> bool: ...
    @property
    def IsFinal(self) -> bool: ...
    @property
    def IsGenericMethod(self) -> bool: ...
    @property
    def IsGenericMethodDefinition(self) -> bool: ...
    @property
    def IsHideBySig(self) -> bool: ...
    @property
    def IsPrivate(self) -> bool: ...
    @property
    def IsPublic(self) -> bool: ...
    @property
    def IsSecurityCritical(self) -> bool: ...
    @property
    def IsSecuritySafeCritical(self) -> bool: ...
    @property
    def IsSecurityTransparent(self) -> bool: ...
    @property
    def IsSpecialName(self) -> bool: ...
    @property
    def IsStatic(self) -> bool: ...
    @property
    def IsVirtual(self) -> bool: ...
    @property
    def MemberType(self) -> MemberTypes: ...
    @property
    def MetadataToken(self) -> int: ...
    @property
    def MethodHandle(self) -> RuntimeMethodHandle: ...
    @property
    def MethodImplementationFlags(self) -> MethodImplAttributes: ...
    @property
    def Module(self) -> Module: ...
    @property
    def Name(self) -> str: ...
    @property
    def ReflectedType(self) -> typing.Type[typing.Any]: ...
    @property
    def ReturnParameter(self) -> ParameterInfo: ...
    @property
    def ReturnType(self) -> typing.Type[typing.Any]: ...
    @property
    def ReturnTypeCustomAttributes(self) -> ICustomAttributeProvider: ...
    def DefineGenericParameters(self, names: Array_1[str]) -> Array_1[GenericTypeParameterBuilder]: ...
    def DefineParameter(self, position: int, attributes: ParameterAttributes, strParamName: str) -> ParameterBuilder: ...
    def SetImplementationFlags(self, attributes: MethodImplAttributes) -> None: ...
    def SetParameters(self, parameterTypes: Array_1[typing.Type[typing.Any]]) -> None: ...
    def SetReturnType(self, returnType: typing.Type[typing.Any]) -> None: ...
    def SetSignature(self, returnType: typing.Type[typing.Any], returnTypeRequiredCustomModifiers: Array_1[typing.Type[typing.Any]], returnTypeOptionalCustomModifiers: Array_1[typing.Type[typing.Any]], parameterTypes: Array_1[typing.Type[typing.Any]], parameterTypeRequiredCustomModifiers: Array_1[Array_1[typing.Type[typing.Any]]], parameterTypeOptionalCustomModifiers: Array_1[Array_1[typing.Type[typing.Any]]]) -> None: ...
    # Skipped GetILGenerator due to it being static, abstract and generic.

    GetILGenerator : GetILGenerator_MethodGroup
    class GetILGenerator_MethodGroup:
        @typing.overload
        def __call__(self) -> ILGenerator:...
        @typing.overload
        def __call__(self, size: int) -> ILGenerator:...

    # Skipped SetCustomAttribute due to it being static, abstract and generic.

    SetCustomAttribute : SetCustomAttribute_MethodGroup
    class SetCustomAttribute_MethodGroup:
        @typing.overload
        def __call__(self, customBuilder: CustomAttributeBuilder) -> None:...
        @typing.overload
        def __call__(self, con: ConstructorInfo, binaryAttribute: Array_1[int]) -> None:...



class ModuleBuilder(Module):
    @property
    def Assembly(self) -> Assembly: ...
    @property
    def CustomAttributes(self) -> IEnumerable_1[CustomAttributeData]: ...
    @property
    def FullyQualifiedName(self) -> str: ...
    @property
    def MDStreamVersion(self) -> int: ...
    @property
    def MetadataToken(self) -> int: ...
    @property
    def ModuleHandle(self) -> ModuleHandle: ...
    @property
    def ModuleVersionId(self) -> Guid: ...
    @property
    def Name(self) -> str: ...
    @property
    def ScopeName(self) -> str: ...
    def CreateGlobalFunctions(self) -> None: ...
    def DefineEnum(self, name: str, visibility: TypeAttributes, underlyingType: typing.Type[typing.Any]) -> EnumBuilder: ...
    def DefineInitializedData(self, name: str, data: Array_1[int], attributes: FieldAttributes) -> FieldBuilder: ...
    def DefineUninitializedData(self, name: str, size: int, attributes: FieldAttributes) -> FieldBuilder: ...
    def GetArrayMethod(self, arrayClass: typing.Type[typing.Any], methodName: str, callingConvention: CallingConventions, returnType: typing.Type[typing.Any], parameterTypes: Array_1[typing.Type[typing.Any]]) -> MethodInfo: ...
    @abc.abstractmethod
    def GetFieldMetadataToken(self, field: FieldInfo) -> int: ...
    @abc.abstractmethod
    def GetSignatureMetadataToken(self, signature: SignatureHelper) -> int: ...
    @abc.abstractmethod
    def GetStringMetadataToken(self, stringConstant: str) -> int: ...
    @abc.abstractmethod
    def GetTypeMetadataToken(self, type: typing.Type[typing.Any]) -> int: ...
    # Skipped DefineGlobalMethod due to it being static, abstract and generic.

    DefineGlobalMethod : DefineGlobalMethod_MethodGroup
    class DefineGlobalMethod_MethodGroup:
        @typing.overload
        def __call__(self, name: str, attributes: MethodAttributes, returnType: typing.Type[typing.Any], parameterTypes: Array_1[typing.Type[typing.Any]]) -> MethodBuilder:...
        @typing.overload
        def __call__(self, name: str, attributes: MethodAttributes, callingConvention: CallingConventions, returnType: typing.Type[typing.Any], parameterTypes: Array_1[typing.Type[typing.Any]]) -> MethodBuilder:...
        @typing.overload
        def __call__(self, name: str, attributes: MethodAttributes, callingConvention: CallingConventions, returnType: typing.Type[typing.Any], requiredReturnTypeCustomModifiers: Array_1[typing.Type[typing.Any]], optionalReturnTypeCustomModifiers: Array_1[typing.Type[typing.Any]], parameterTypes: Array_1[typing.Type[typing.Any]], requiredParameterTypeCustomModifiers: Array_1[Array_1[typing.Type[typing.Any]]], optionalParameterTypeCustomModifiers: Array_1[Array_1[typing.Type[typing.Any]]]) -> MethodBuilder:...

    # Skipped DefinePInvokeMethod due to it being static, abstract and generic.

    DefinePInvokeMethod : DefinePInvokeMethod_MethodGroup
    class DefinePInvokeMethod_MethodGroup:
        @typing.overload
        def __call__(self, name: str, dllName: str, attributes: MethodAttributes, callingConvention: CallingConventions, returnType: typing.Type[typing.Any], parameterTypes: Array_1[typing.Type[typing.Any]], nativeCallConv: CallingConvention, nativeCharSet: CharSet) -> MethodBuilder:...
        @typing.overload
        def __call__(self, name: str, dllName: str, entryName: str, attributes: MethodAttributes, callingConvention: CallingConventions, returnType: typing.Type[typing.Any], parameterTypes: Array_1[typing.Type[typing.Any]], nativeCallConv: CallingConvention, nativeCharSet: CharSet) -> MethodBuilder:...

    # Skipped DefineType due to it being static, abstract and generic.

    DefineType : DefineType_MethodGroup
    class DefineType_MethodGroup:
        @typing.overload
        def __call__(self, name: str) -> TypeBuilder:...
        @typing.overload
        def __call__(self, name: str, attr: TypeAttributes) -> TypeBuilder:...
        @typing.overload
        def __call__(self, name: str, attr: TypeAttributes, parent: typing.Type[typing.Any]) -> TypeBuilder:...
        @typing.overload
        def __call__(self, name: str, attr: TypeAttributes, parent: typing.Type[typing.Any], typesize: int) -> TypeBuilder:...
        @typing.overload
        def __call__(self, name: str, attr: TypeAttributes, parent: typing.Type[typing.Any], interfaces: Array_1[typing.Type[typing.Any]]) -> TypeBuilder:...
        @typing.overload
        def __call__(self, name: str, attr: TypeAttributes, parent: typing.Type[typing.Any], packsize: PackingSize) -> TypeBuilder:...
        @typing.overload
        def __call__(self, name: str, attr: TypeAttributes, parent: typing.Type[typing.Any], packingSize: PackingSize, typesize: int) -> TypeBuilder:...

    # Skipped GetMethodMetadataToken due to it being static, abstract and generic.

    GetMethodMetadataToken : GetMethodMetadataToken_MethodGroup
    class GetMethodMetadataToken_MethodGroup:
        @typing.overload
        def __call__(self, constructor: ConstructorInfo) -> int:...
        @typing.overload
        def __call__(self, method: MethodInfo) -> int:...

    # Skipped SetCustomAttribute due to it being static, abstract and generic.

    SetCustomAttribute : SetCustomAttribute_MethodGroup
    class SetCustomAttribute_MethodGroup:
        @typing.overload
        def __call__(self, customBuilder: CustomAttributeBuilder) -> None:...
        @typing.overload
        def __call__(self, con: ConstructorInfo, binaryAttribute: Array_1[int]) -> None:...



class OpCode(IEquatable_1[OpCode]):
    @property
    def FlowControl(self) -> FlowControl: ...
    @property
    def Name(self) -> str: ...
    @property
    def OpCodeType(self) -> OpCodeType: ...
    @property
    def OperandType(self) -> OperandType: ...
    @property
    def Size(self) -> int: ...
    @property
    def StackBehaviourPop(self) -> StackBehaviour: ...
    @property
    def StackBehaviourPush(self) -> StackBehaviour: ...
    @property
    def Value(self) -> int: ...
    def GetHashCode(self) -> int: ...
    def __eq__(self, a: OpCode, b: OpCode) -> bool: ...
    def __ne__(self, a: OpCode, b: OpCode) -> bool: ...
    def ToString(self) -> str: ...
    # Skipped Equals due to it being static, abstract and generic.

    Equals : Equals_MethodGroup
    class Equals_MethodGroup:
        @typing.overload
        def __call__(self, obj: OpCode) -> bool:...
        @typing.overload
        def __call__(self, obj: typing.Any) -> bool:...



class OpCodes:
    Add : OpCode
    Add_Ovf : OpCode
    Add_Ovf_Un : OpCode
    And : OpCode
    Arglist : OpCode
    Beq : OpCode
    Beq_S : OpCode
    Bge : OpCode
    Bge_S : OpCode
    Bge_Un : OpCode
    Bge_Un_S : OpCode
    Bgt : OpCode
    Bgt_S : OpCode
    Bgt_Un : OpCode
    Bgt_Un_S : OpCode
    Ble : OpCode
    Ble_S : OpCode
    Ble_Un : OpCode
    Ble_Un_S : OpCode
    Blt : OpCode
    Blt_S : OpCode
    Blt_Un : OpCode
    Blt_Un_S : OpCode
    Bne_Un : OpCode
    Bne_Un_S : OpCode
    Box : OpCode
    Br : OpCode
    Br_S : OpCode
    Break : OpCode
    Brfalse : OpCode
    Brfalse_S : OpCode
    Brtrue : OpCode
    Brtrue_S : OpCode
    Call : OpCode
    Calli : OpCode
    Callvirt : OpCode
    Castclass : OpCode
    Ceq : OpCode
    Cgt : OpCode
    Cgt_Un : OpCode
    Ckfinite : OpCode
    Clt : OpCode
    Clt_Un : OpCode
    Constrained : OpCode
    Conv_I : OpCode
    Conv_I1 : OpCode
    Conv_I2 : OpCode
    Conv_I4 : OpCode
    Conv_I8 : OpCode
    Conv_Ovf_I : OpCode
    Conv_Ovf_I_Un : OpCode
    Conv_Ovf_I1 : OpCode
    Conv_Ovf_I1_Un : OpCode
    Conv_Ovf_I2 : OpCode
    Conv_Ovf_I2_Un : OpCode
    Conv_Ovf_I4 : OpCode
    Conv_Ovf_I4_Un : OpCode
    Conv_Ovf_I8 : OpCode
    Conv_Ovf_I8_Un : OpCode
    Conv_Ovf_U : OpCode
    Conv_Ovf_U_Un : OpCode
    Conv_Ovf_U1 : OpCode
    Conv_Ovf_U1_Un : OpCode
    Conv_Ovf_U2 : OpCode
    Conv_Ovf_U2_Un : OpCode
    Conv_Ovf_U4 : OpCode
    Conv_Ovf_U4_Un : OpCode
    Conv_Ovf_U8 : OpCode
    Conv_Ovf_U8_Un : OpCode
    Conv_R_Un : OpCode
    Conv_R4 : OpCode
    Conv_R8 : OpCode
    Conv_U : OpCode
    Conv_U1 : OpCode
    Conv_U2 : OpCode
    Conv_U4 : OpCode
    Conv_U8 : OpCode
    Cpblk : OpCode
    Cpobj : OpCode
    Div : OpCode
    Div_Un : OpCode
    Dup : OpCode
    Endfilter : OpCode
    Endfinally : OpCode
    Initblk : OpCode
    Initobj : OpCode
    Isinst : OpCode
    Jmp : OpCode
    Ldarg : OpCode
    Ldarg_0 : OpCode
    Ldarg_1 : OpCode
    Ldarg_2 : OpCode
    Ldarg_3 : OpCode
    Ldarg_S : OpCode
    Ldarga : OpCode
    Ldarga_S : OpCode
    Ldc_I4 : OpCode
    Ldc_I4_0 : OpCode
    Ldc_I4_1 : OpCode
    Ldc_I4_2 : OpCode
    Ldc_I4_3 : OpCode
    Ldc_I4_4 : OpCode
    Ldc_I4_5 : OpCode
    Ldc_I4_6 : OpCode
    Ldc_I4_7 : OpCode
    Ldc_I4_8 : OpCode
    Ldc_I4_M1 : OpCode
    Ldc_I4_S : OpCode
    Ldc_I8 : OpCode
    Ldc_R4 : OpCode
    Ldc_R8 : OpCode
    Ldelem : OpCode
    Ldelem_I : OpCode
    Ldelem_I1 : OpCode
    Ldelem_I2 : OpCode
    Ldelem_I4 : OpCode
    Ldelem_I8 : OpCode
    Ldelem_R4 : OpCode
    Ldelem_R8 : OpCode
    Ldelem_Ref : OpCode
    Ldelem_U1 : OpCode
    Ldelem_U2 : OpCode
    Ldelem_U4 : OpCode
    Ldelema : OpCode
    Ldfld : OpCode
    Ldflda : OpCode
    Ldftn : OpCode
    Ldind_I : OpCode
    Ldind_I1 : OpCode
    Ldind_I2 : OpCode
    Ldind_I4 : OpCode
    Ldind_I8 : OpCode
    Ldind_R4 : OpCode
    Ldind_R8 : OpCode
    Ldind_Ref : OpCode
    Ldind_U1 : OpCode
    Ldind_U2 : OpCode
    Ldind_U4 : OpCode
    Ldlen : OpCode
    Ldloc : OpCode
    Ldloc_0 : OpCode
    Ldloc_1 : OpCode
    Ldloc_2 : OpCode
    Ldloc_3 : OpCode
    Ldloc_S : OpCode
    Ldloca : OpCode
    Ldloca_S : OpCode
    Ldnull : OpCode
    Ldobj : OpCode
    Ldsfld : OpCode
    Ldsflda : OpCode
    Ldstr : OpCode
    Ldtoken : OpCode
    Ldvirtftn : OpCode
    Leave : OpCode
    Leave_S : OpCode
    Localloc : OpCode
    Mkrefany : OpCode
    Mul : OpCode
    Mul_Ovf : OpCode
    Mul_Ovf_Un : OpCode
    Neg : OpCode
    Newarr : OpCode
    Newobj : OpCode
    Nop : OpCode
    Not : OpCode
    Or : OpCode
    Pop : OpCode
    Prefix1 : OpCode
    Prefix2 : OpCode
    Prefix3 : OpCode
    Prefix4 : OpCode
    Prefix5 : OpCode
    Prefix6 : OpCode
    Prefix7 : OpCode
    Prefixref : OpCode
    Readonly : OpCode
    Refanytype : OpCode
    Refanyval : OpCode
    Rem : OpCode
    Rem_Un : OpCode
    Ret : OpCode
    Rethrow : OpCode
    Shl : OpCode
    Shr : OpCode
    Shr_Un : OpCode
    Sizeof : OpCode
    Starg : OpCode
    Starg_S : OpCode
    Stelem : OpCode
    Stelem_I : OpCode
    Stelem_I1 : OpCode
    Stelem_I2 : OpCode
    Stelem_I4 : OpCode
    Stelem_I8 : OpCode
    Stelem_R4 : OpCode
    Stelem_R8 : OpCode
    Stelem_Ref : OpCode
    Stfld : OpCode
    Stind_I : OpCode
    Stind_I1 : OpCode
    Stind_I2 : OpCode
    Stind_I4 : OpCode
    Stind_I8 : OpCode
    Stind_R4 : OpCode
    Stind_R8 : OpCode
    Stind_Ref : OpCode
    Stloc : OpCode
    Stloc_0 : OpCode
    Stloc_1 : OpCode
    Stloc_2 : OpCode
    Stloc_3 : OpCode
    Stloc_S : OpCode
    Stobj : OpCode
    Stsfld : OpCode
    Sub : OpCode
    Sub_Ovf : OpCode
    Sub_Ovf_Un : OpCode
    Switch : OpCode
    Tailcall : OpCode
    Throw : OpCode
    Unaligned : OpCode
    Unbox : OpCode
    Unbox_Any : OpCode
    Volatile : OpCode
    Xor : OpCode
    @staticmethod
    def TakesSingleByteArgument(inst: OpCode) -> bool: ...


class OpCodeType(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    Annotation : OpCodeType # 0
    Macro : OpCodeType # 1
    Nternal : OpCodeType # 2
    Objmodel : OpCodeType # 3
    Prefix : OpCodeType # 4
    Primitive : OpCodeType # 5


class OperandType(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    InlineBrTarget : OperandType # 0
    InlineField : OperandType # 1
    InlineI : OperandType # 2
    InlineI8 : OperandType # 3
    InlineMethod : OperandType # 4
    InlineNone : OperandType # 5
    InlinePhi : OperandType # 6
    InlineR : OperandType # 7
    InlineSig : OperandType # 9
    InlineString : OperandType # 10
    InlineSwitch : OperandType # 11
    InlineTok : OperandType # 12
    InlineType : OperandType # 13
    InlineVar : OperandType # 14
    ShortInlineBrTarget : OperandType # 15
    ShortInlineI : OperandType # 16
    ShortInlineR : OperandType # 17
    ShortInlineVar : OperandType # 18


class PackingSize(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    Unspecified : PackingSize # 0
    Size1 : PackingSize # 1
    Size2 : PackingSize # 2
    Size4 : PackingSize # 4
    Size8 : PackingSize # 8
    Size16 : PackingSize # 16
    Size32 : PackingSize # 32
    Size64 : PackingSize # 64
    Size128 : PackingSize # 128


class ParameterBuilder(abc.ABC):
    @property
    def Attributes(self) -> int: ...
    @property
    def IsIn(self) -> bool: ...
    @property
    def IsOptional(self) -> bool: ...
    @property
    def IsOut(self) -> bool: ...
    @property
    def Name(self) -> str: ...
    @property
    def Position(self) -> int: ...
    def SetConstant(self, defaultValue: typing.Any) -> None: ...
    # Skipped SetCustomAttribute due to it being static, abstract and generic.

    SetCustomAttribute : SetCustomAttribute_MethodGroup
    class SetCustomAttribute_MethodGroup:
        @typing.overload
        def __call__(self, customBuilder: CustomAttributeBuilder) -> None:...
        @typing.overload
        def __call__(self, con: ConstructorInfo, binaryAttribute: Array_1[int]) -> None:...



class PEFileKinds(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    Dll : PEFileKinds # 1
    ConsoleApplication : PEFileKinds # 2
    WindowApplication : PEFileKinds # 3


class PropertyBuilder(PropertyInfo):
    @property
    def Attributes(self) -> PropertyAttributes: ...
    @property
    def CanRead(self) -> bool: ...
    @property
    def CanWrite(self) -> bool: ...
    @property
    def CustomAttributes(self) -> IEnumerable_1[CustomAttributeData]: ...
    @property
    def DeclaringType(self) -> typing.Type[typing.Any]: ...
    @property
    def GetMethod(self) -> MethodInfo: ...
    @property
    def IsCollectible(self) -> bool: ...
    @property
    def IsSpecialName(self) -> bool: ...
    @property
    def MemberType(self) -> MemberTypes: ...
    @property
    def MetadataToken(self) -> int: ...
    @property
    def Module(self) -> Module: ...
    @property
    def Name(self) -> str: ...
    @property
    def PropertyType(self) -> typing.Type[typing.Any]: ...
    @property
    def ReflectedType(self) -> typing.Type[typing.Any]: ...
    @property
    def SetMethod(self) -> MethodInfo: ...
    def AddOtherMethod(self, mdBuilder: MethodBuilder) -> None: ...
    def SetConstant(self, defaultValue: typing.Any) -> None: ...
    def SetGetMethod(self, mdBuilder: MethodBuilder) -> None: ...
    def SetSetMethod(self, mdBuilder: MethodBuilder) -> None: ...
    # Skipped SetCustomAttribute due to it being static, abstract and generic.

    SetCustomAttribute : SetCustomAttribute_MethodGroup
    class SetCustomAttribute_MethodGroup:
        @typing.overload
        def __call__(self, customBuilder: CustomAttributeBuilder) -> None:...
        @typing.overload
        def __call__(self, con: ConstructorInfo, binaryAttribute: Array_1[int]) -> None:...



class SignatureHelper:
    def AddArguments(self, arguments: Array_1[typing.Type[typing.Any]], requiredCustomModifiers: Array_1[Array_1[typing.Type[typing.Any]]], optionalCustomModifiers: Array_1[Array_1[typing.Type[typing.Any]]]) -> None: ...
    def AddSentinel(self) -> None: ...
    def Equals(self, obj: typing.Any) -> bool: ...
    @staticmethod
    def GetFieldSigHelper(mod: Module) -> SignatureHelper: ...
    def GetHashCode(self) -> int: ...
    def GetSignature(self) -> Array_1[int]: ...
    def ToString(self) -> str: ...
    # Skipped AddArgument due to it being static, abstract and generic.

    AddArgument : AddArgument_MethodGroup
    class AddArgument_MethodGroup:
        @typing.overload
        def __call__(self, clsArgument: typing.Type[typing.Any]) -> None:...
        @typing.overload
        def __call__(self, argument: typing.Type[typing.Any], pinned: bool) -> None:...
        @typing.overload
        def __call__(self, argument: typing.Type[typing.Any], requiredCustomModifiers: Array_1[typing.Type[typing.Any]], optionalCustomModifiers: Array_1[typing.Type[typing.Any]]) -> None:...

    # Skipped GetLocalVarSigHelper due to it being static, abstract and generic.

    GetLocalVarSigHelper : GetLocalVarSigHelper_MethodGroup
    class GetLocalVarSigHelper_MethodGroup:
        @typing.overload
        def __call__(self) -> SignatureHelper:...
        @typing.overload
        def __call__(self, mod: Module) -> SignatureHelper:...

    # Skipped GetMethodSigHelper due to it being static, abstract and generic.

    GetMethodSigHelper : GetMethodSigHelper_MethodGroup
    class GetMethodSigHelper_MethodGroup:
        @typing.overload
        def __call__(self, callingConvention: CallingConventions, returnType: typing.Type[typing.Any]) -> SignatureHelper:...
        @typing.overload
        def __call__(self, mod: Module, callingConvention: CallingConventions, returnType: typing.Type[typing.Any]) -> SignatureHelper:...
        @typing.overload
        def __call__(self, mod: Module, returnType: typing.Type[typing.Any], parameterTypes: Array_1[typing.Type[typing.Any]]) -> SignatureHelper:...

    # Skipped GetPropertySigHelper due to it being static, abstract and generic.

    GetPropertySigHelper : GetPropertySigHelper_MethodGroup
    class GetPropertySigHelper_MethodGroup:
        @typing.overload
        def __call__(self, mod: Module, returnType: typing.Type[typing.Any], parameterTypes: Array_1[typing.Type[typing.Any]]) -> SignatureHelper:...
        @typing.overload
        def __call__(self, mod: Module, returnType: typing.Type[typing.Any], requiredReturnTypeCustomModifiers: Array_1[typing.Type[typing.Any]], optionalReturnTypeCustomModifiers: Array_1[typing.Type[typing.Any]], parameterTypes: Array_1[typing.Type[typing.Any]], requiredParameterTypeCustomModifiers: Array_1[Array_1[typing.Type[typing.Any]]], optionalParameterTypeCustomModifiers: Array_1[Array_1[typing.Type[typing.Any]]]) -> SignatureHelper:...
        @typing.overload
        def __call__(self, mod: Module, callingConvention: CallingConventions, returnType: typing.Type[typing.Any], requiredReturnTypeCustomModifiers: Array_1[typing.Type[typing.Any]], optionalReturnTypeCustomModifiers: Array_1[typing.Type[typing.Any]], parameterTypes: Array_1[typing.Type[typing.Any]], requiredParameterTypeCustomModifiers: Array_1[Array_1[typing.Type[typing.Any]]], optionalParameterTypeCustomModifiers: Array_1[Array_1[typing.Type[typing.Any]]]) -> SignatureHelper:...



class StackBehaviour(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    Pop0 : StackBehaviour # 0
    Pop1 : StackBehaviour # 1
    Pop1_pop1 : StackBehaviour # 2
    Popi : StackBehaviour # 3
    Popi_pop1 : StackBehaviour # 4
    Popi_popi : StackBehaviour # 5
    Popi_popi8 : StackBehaviour # 6
    Popi_popi_popi : StackBehaviour # 7
    Popi_popr4 : StackBehaviour # 8
    Popi_popr8 : StackBehaviour # 9
    Popref : StackBehaviour # 10
    Popref_pop1 : StackBehaviour # 11
    Popref_popi : StackBehaviour # 12
    Popref_popi_popi : StackBehaviour # 13
    Popref_popi_popi8 : StackBehaviour # 14
    Popref_popi_popr4 : StackBehaviour # 15
    Popref_popi_popr8 : StackBehaviour # 16
    Popref_popi_popref : StackBehaviour # 17
    Push0 : StackBehaviour # 18
    Push1 : StackBehaviour # 19
    Push1_push1 : StackBehaviour # 20
    Pushi : StackBehaviour # 21
    Pushi8 : StackBehaviour # 22
    Pushr4 : StackBehaviour # 23
    Pushr8 : StackBehaviour # 24
    Pushref : StackBehaviour # 25
    Varpop : StackBehaviour # 26
    Varpush : StackBehaviour # 27
    Popref_popi_pop1 : StackBehaviour # 28


class TypeBuilder(TypeInfo):
    UnspecifiedTypeSize : int
    @property
    def Assembly(self) -> Assembly: ...
    @property
    def AssemblyQualifiedName(self) -> str: ...
    @property
    def Attributes(self) -> TypeAttributes: ...
    @property
    def BaseType(self) -> typing.Type[typing.Any]: ...
    @property
    def ContainsGenericParameters(self) -> bool: ...
    @property
    def CustomAttributes(self) -> IEnumerable_1[CustomAttributeData]: ...
    @property
    def DeclaredConstructors(self) -> IEnumerable_1[ConstructorInfo]: ...
    @property
    def DeclaredEvents(self) -> IEnumerable_1[EventInfo]: ...
    @property
    def DeclaredFields(self) -> IEnumerable_1[FieldInfo]: ...
    @property
    def DeclaredMembers(self) -> IEnumerable_1[MemberInfo]: ...
    @property
    def DeclaredMethods(self) -> IEnumerable_1[MethodInfo]: ...
    @property
    def DeclaredNestedTypes(self) -> IEnumerable_1[TypeInfo]: ...
    @property
    def DeclaredProperties(self) -> IEnumerable_1[PropertyInfo]: ...
    @property
    def DeclaringMethod(self) -> MethodBase: ...
    @property
    def DeclaringType(self) -> typing.Type[typing.Any]: ...
    @property
    def FullName(self) -> str: ...
    @property
    def GenericParameterAttributes(self) -> GenericParameterAttributes: ...
    @property
    def GenericParameterPosition(self) -> int: ...
    @property
    def GenericTypeArguments(self) -> Array_1[typing.Type[typing.Any]]: ...
    @property
    def GenericTypeParameters(self) -> Array_1[typing.Type[typing.Any]]: ...
    @property
    def GUID(self) -> Guid: ...
    @property
    def HasElementType(self) -> bool: ...
    @property
    def ImplementedInterfaces(self) -> IEnumerable_1[typing.Type[typing.Any]]: ...
    @property
    def IsAbstract(self) -> bool: ...
    @property
    def IsAnsiClass(self) -> bool: ...
    @property
    def IsArray(self) -> bool: ...
    @property
    def IsAutoClass(self) -> bool: ...
    @property
    def IsAutoLayout(self) -> bool: ...
    @property
    def IsByRef(self) -> bool: ...
    @property
    def IsByRefLike(self) -> bool: ...
    @property
    def IsClass(self) -> bool: ...
    @property
    def IsCollectible(self) -> bool: ...
    @property
    def IsCOMObject(self) -> bool: ...
    @property
    def IsConstructedGenericType(self) -> bool: ...
    @property
    def IsContextful(self) -> bool: ...
    @property
    def IsEnum(self) -> bool: ...
    @property
    def IsExplicitLayout(self) -> bool: ...
    @property
    def IsFunctionPointer(self) -> bool: ...
    @property
    def IsGenericMethodParameter(self) -> bool: ...
    @property
    def IsGenericParameter(self) -> bool: ...
    @property
    def IsGenericType(self) -> bool: ...
    @property
    def IsGenericTypeDefinition(self) -> bool: ...
    @property
    def IsGenericTypeParameter(self) -> bool: ...
    @property
    def IsImport(self) -> bool: ...
    @property
    def IsInterface(self) -> bool: ...
    @property
    def IsLayoutSequential(self) -> bool: ...
    @property
    def IsMarshalByRef(self) -> bool: ...
    @property
    def IsNested(self) -> bool: ...
    @property
    def IsNestedAssembly(self) -> bool: ...
    @property
    def IsNestedFamANDAssem(self) -> bool: ...
    @property
    def IsNestedFamily(self) -> bool: ...
    @property
    def IsNestedFamORAssem(self) -> bool: ...
    @property
    def IsNestedPrivate(self) -> bool: ...
    @property
    def IsNestedPublic(self) -> bool: ...
    @property
    def IsNotPublic(self) -> bool: ...
    @property
    def IsPointer(self) -> bool: ...
    @property
    def IsPrimitive(self) -> bool: ...
    @property
    def IsPublic(self) -> bool: ...
    @property
    def IsSealed(self) -> bool: ...
    @property
    def IsSecurityCritical(self) -> bool: ...
    @property
    def IsSecuritySafeCritical(self) -> bool: ...
    @property
    def IsSecurityTransparent(self) -> bool: ...
    @property
    def IsSerializable(self) -> bool: ...
    @property
    def IsSignatureType(self) -> bool: ...
    @property
    def IsSpecialName(self) -> bool: ...
    @property
    def IsSZArray(self) -> bool: ...
    @property
    def IsTypeDefinition(self) -> bool: ...
    @property
    def IsUnicodeClass(self) -> bool: ...
    @property
    def IsUnmanagedFunctionPointer(self) -> bool: ...
    @property
    def IsValueType(self) -> bool: ...
    @property
    def IsVariableBoundArray(self) -> bool: ...
    @property
    def IsVisible(self) -> bool: ...
    @property
    def MemberType(self) -> MemberTypes: ...
    @property
    def MetadataToken(self) -> int: ...
    @property
    def Module(self) -> Module: ...
    @property
    def Name(self) -> str: ...
    @property
    def Namespace(self) -> str: ...
    @property
    def PackingSize(self) -> PackingSize: ...
    @property
    def ReflectedType(self) -> typing.Type[typing.Any]: ...
    @property
    def Size(self) -> int: ...
    @property
    def StructLayoutAttribute(self) -> StructLayoutAttribute: ...
    @property
    def TypeHandle(self) -> RuntimeTypeHandle: ...
    @property
    def TypeInitializer(self) -> ConstructorInfo: ...
    @property
    def UnderlyingSystemType(self) -> typing.Type[typing.Any]: ...
    def AddInterfaceImplementation(self, interfaceType: typing.Type[typing.Any]) -> None: ...
    def CreateType(self) -> typing.Type[typing.Any]: ...
    def CreateTypeInfo(self) -> TypeInfo: ...
    def DefineDefaultConstructor(self, attributes: MethodAttributes) -> ConstructorBuilder: ...
    def DefineEvent(self, name: str, attributes: EventAttributes, eventtype: typing.Type[typing.Any]) -> EventBuilder: ...
    def DefineGenericParameters(self, names: Array_1[str]) -> Array_1[GenericTypeParameterBuilder]: ...
    def DefineInitializedData(self, name: str, data: Array_1[int], attributes: FieldAttributes) -> FieldBuilder: ...
    def DefineMethodOverride(self, methodInfoBody: MethodInfo, methodInfoDeclaration: MethodInfo) -> None: ...
    def DefineTypeInitializer(self) -> ConstructorBuilder: ...
    def DefineUninitializedData(self, name: str, size: int, attributes: FieldAttributes) -> FieldBuilder: ...
    @staticmethod
    def GetConstructor(type: typing.Type[typing.Any], constructor: ConstructorInfo) -> ConstructorInfo: ...
    @staticmethod
    def GetField(type: typing.Type[typing.Any], field: FieldInfo) -> FieldInfo: ...
    @staticmethod
    def GetMethod(type: typing.Type[typing.Any], method: MethodInfo) -> MethodInfo: ...
    def IsCreated(self) -> bool: ...
    def MakeByRefType(self) -> typing.Type[typing.Any]: ...
    def MakeGenericType(self, typeArguments: Array_1[typing.Type[typing.Any]]) -> typing.Type[typing.Any]: ...
    def MakePointerType(self) -> typing.Type[typing.Any]: ...
    def SetParent(self, parent: typing.Type[typing.Any]) -> None: ...
    # Skipped DefineConstructor due to it being static, abstract and generic.

    DefineConstructor : DefineConstructor_MethodGroup
    class DefineConstructor_MethodGroup:
        @typing.overload
        def __call__(self, attributes: MethodAttributes, callingConvention: CallingConventions, parameterTypes: Array_1[typing.Type[typing.Any]]) -> ConstructorBuilder:...
        @typing.overload
        def __call__(self, attributes: MethodAttributes, callingConvention: CallingConventions, parameterTypes: Array_1[typing.Type[typing.Any]], requiredCustomModifiers: Array_1[Array_1[typing.Type[typing.Any]]], optionalCustomModifiers: Array_1[Array_1[typing.Type[typing.Any]]]) -> ConstructorBuilder:...

    # Skipped DefineField due to it being static, abstract and generic.

    DefineField : DefineField_MethodGroup
    class DefineField_MethodGroup:
        @typing.overload
        def __call__(self, fieldName: str, type: typing.Type[typing.Any], attributes: FieldAttributes) -> FieldBuilder:...
        @typing.overload
        def __call__(self, fieldName: str, type: typing.Type[typing.Any], requiredCustomModifiers: Array_1[typing.Type[typing.Any]], optionalCustomModifiers: Array_1[typing.Type[typing.Any]], attributes: FieldAttributes) -> FieldBuilder:...

    # Skipped DefineMethod due to it being static, abstract and generic.

    DefineMethod : DefineMethod_MethodGroup
    class DefineMethod_MethodGroup:
        @typing.overload
        def __call__(self, name: str, attributes: MethodAttributes) -> MethodBuilder:...
        @typing.overload
        def __call__(self, name: str, attributes: MethodAttributes, callingConvention: CallingConventions) -> MethodBuilder:...
        @typing.overload
        def __call__(self, name: str, attributes: MethodAttributes, returnType: typing.Type[typing.Any], parameterTypes: Array_1[typing.Type[typing.Any]]) -> MethodBuilder:...
        @typing.overload
        def __call__(self, name: str, attributes: MethodAttributes, callingConvention: CallingConventions, returnType: typing.Type[typing.Any], parameterTypes: Array_1[typing.Type[typing.Any]]) -> MethodBuilder:...
        @typing.overload
        def __call__(self, name: str, attributes: MethodAttributes, callingConvention: CallingConventions, returnType: typing.Type[typing.Any], returnTypeRequiredCustomModifiers: Array_1[typing.Type[typing.Any]], returnTypeOptionalCustomModifiers: Array_1[typing.Type[typing.Any]], parameterTypes: Array_1[typing.Type[typing.Any]], parameterTypeRequiredCustomModifiers: Array_1[Array_1[typing.Type[typing.Any]]], parameterTypeOptionalCustomModifiers: Array_1[Array_1[typing.Type[typing.Any]]]) -> MethodBuilder:...

    # Skipped DefineNestedType due to it being static, abstract and generic.

    DefineNestedType : DefineNestedType_MethodGroup
    class DefineNestedType_MethodGroup:
        @typing.overload
        def __call__(self, name: str) -> TypeBuilder:...
        @typing.overload
        def __call__(self, name: str, attr: TypeAttributes) -> TypeBuilder:...
        @typing.overload
        def __call__(self, name: str, attr: TypeAttributes, parent: typing.Type[typing.Any]) -> TypeBuilder:...
        @typing.overload
        def __call__(self, name: str, attr: TypeAttributes, parent: typing.Type[typing.Any], typeSize: int) -> TypeBuilder:...
        @typing.overload
        def __call__(self, name: str, attr: TypeAttributes, parent: typing.Type[typing.Any], interfaces: Array_1[typing.Type[typing.Any]]) -> TypeBuilder:...
        @typing.overload
        def __call__(self, name: str, attr: TypeAttributes, parent: typing.Type[typing.Any], packSize: PackingSize) -> TypeBuilder:...
        @typing.overload
        def __call__(self, name: str, attr: TypeAttributes, parent: typing.Type[typing.Any], packSize: PackingSize, typeSize: int) -> TypeBuilder:...

    # Skipped DefinePInvokeMethod due to it being static, abstract and generic.

    DefinePInvokeMethod : DefinePInvokeMethod_MethodGroup
    class DefinePInvokeMethod_MethodGroup:
        @typing.overload
        def __call__(self, name: str, dllName: str, attributes: MethodAttributes, callingConvention: CallingConventions, returnType: typing.Type[typing.Any], parameterTypes: Array_1[typing.Type[typing.Any]], nativeCallConv: CallingConvention, nativeCharSet: CharSet) -> MethodBuilder:...
        @typing.overload
        def __call__(self, name: str, dllName: str, entryName: str, attributes: MethodAttributes, callingConvention: CallingConventions, returnType: typing.Type[typing.Any], parameterTypes: Array_1[typing.Type[typing.Any]], nativeCallConv: CallingConvention, nativeCharSet: CharSet) -> MethodBuilder:...
        @typing.overload
        def __call__(self, name: str, dllName: str, entryName: str, attributes: MethodAttributes, callingConvention: CallingConventions, returnType: typing.Type[typing.Any], returnTypeRequiredCustomModifiers: Array_1[typing.Type[typing.Any]], returnTypeOptionalCustomModifiers: Array_1[typing.Type[typing.Any]], parameterTypes: Array_1[typing.Type[typing.Any]], parameterTypeRequiredCustomModifiers: Array_1[Array_1[typing.Type[typing.Any]]], parameterTypeOptionalCustomModifiers: Array_1[Array_1[typing.Type[typing.Any]]], nativeCallConv: CallingConvention, nativeCharSet: CharSet) -> MethodBuilder:...

    # Skipped DefineProperty due to it being static, abstract and generic.

    DefineProperty : DefineProperty_MethodGroup
    class DefineProperty_MethodGroup:
        @typing.overload
        def __call__(self, name: str, attributes: PropertyAttributes, returnType: typing.Type[typing.Any], parameterTypes: Array_1[typing.Type[typing.Any]]) -> PropertyBuilder:...
        @typing.overload
        def __call__(self, name: str, attributes: PropertyAttributes, callingConvention: CallingConventions, returnType: typing.Type[typing.Any], parameterTypes: Array_1[typing.Type[typing.Any]]) -> PropertyBuilder:...
        @typing.overload
        def __call__(self, name: str, attributes: PropertyAttributes, returnType: typing.Type[typing.Any], returnTypeRequiredCustomModifiers: Array_1[typing.Type[typing.Any]], returnTypeOptionalCustomModifiers: Array_1[typing.Type[typing.Any]], parameterTypes: Array_1[typing.Type[typing.Any]], parameterTypeRequiredCustomModifiers: Array_1[Array_1[typing.Type[typing.Any]]], parameterTypeOptionalCustomModifiers: Array_1[Array_1[typing.Type[typing.Any]]]) -> PropertyBuilder:...
        @typing.overload
        def __call__(self, name: str, attributes: PropertyAttributes, callingConvention: CallingConventions, returnType: typing.Type[typing.Any], returnTypeRequiredCustomModifiers: Array_1[typing.Type[typing.Any]], returnTypeOptionalCustomModifiers: Array_1[typing.Type[typing.Any]], parameterTypes: Array_1[typing.Type[typing.Any]], parameterTypeRequiredCustomModifiers: Array_1[Array_1[typing.Type[typing.Any]]], parameterTypeOptionalCustomModifiers: Array_1[Array_1[typing.Type[typing.Any]]]) -> PropertyBuilder:...

    # Skipped MakeArrayType due to it being static, abstract and generic.

    MakeArrayType : MakeArrayType_MethodGroup
    class MakeArrayType_MethodGroup:
        @typing.overload
        def __call__(self) -> typing.Type[typing.Any]:...
        @typing.overload
        def __call__(self, rank: int) -> typing.Type[typing.Any]:...

    # Skipped SetCustomAttribute due to it being static, abstract and generic.

    SetCustomAttribute : SetCustomAttribute_MethodGroup
    class SetCustomAttribute_MethodGroup:
        @typing.overload
        def __call__(self, customBuilder: CustomAttributeBuilder) -> None:...
        @typing.overload
        def __call__(self, con: ConstructorInfo, binaryAttribute: Array_1[int]) -> None:...





from typing import Any
import numpy as np
import pandas as pd
import inspect
import enum

from .utils import (SimComponent, SimDoubleParameter, SimIntegerParameter, SimStringParameter,
                    SimBoolParameter, SimEnumParameter, SimMultiValueField3D, SimMultiValueBigTable, FileInfo, DirectoryInfo,
                    set_property_to_sim_component, set_property_to_parameter, set_property_to_value_field,
                    set_property_to_file_info, set_property_to_list, set_property_to_dict, set_property_to_directory_info)

from .simultan_object import SimultanObject, MetaMock

from .default_types import ComponentList, ComponentDictionary

from SIMULTAN.Data.Components import (ComponentWalker, SimComponent, SimBoolParameter, SimDoubleParameter,
                                      SimEnumParameter, SimIntegerParameter, SimStringParameter, ComponentMapping,
                                      SimSlot, SimComponentVisibility, SimChildComponentEntry, SimDefaultSlots,
                                      SimParameterOperations, SimComponentReference)


class TypeSetterFcnLookupDict(object):
    lookup_dict = {None: lambda x: None,
                   SimComponent: set_property_to_sim_component,
                   SimultanObject: set_property_to_sim_component,
                   SimDoubleParameter: set_property_to_parameter,
                   SimIntegerParameter: set_property_to_parameter,
                   SimStringParameter: set_property_to_parameter,
                   SimBoolParameter: set_property_to_parameter,
                   SimEnumParameter: set_property_to_parameter,
                   SimMultiValueField3D: set_property_to_value_field,
                   SimMultiValueBigTable: set_property_to_value_field,
                   int: set_property_to_parameter,
                   np.int32: set_property_to_parameter,
                   np.int64: set_property_to_parameter,
                   float: set_property_to_parameter,
                   np.float32: set_property_to_parameter,
                   np.float64: set_property_to_parameter,
                   str: set_property_to_parameter,
                   bool: set_property_to_parameter,
                   FileInfo: set_property_to_file_info,
                   DirectoryInfo: set_property_to_directory_info,
                   list: set_property_to_list,
                   tuple: set_property_to_list,
                   set: set_property_to_list,
                   dict: set_property_to_dict,
                   enum.Enum: set_property_to_parameter,
                   enum.IntEnum: set_property_to_parameter,
                   enum.EnumType: set_property_to_parameter,
                   ComponentDictionary: set_property_to_sim_component,
                   ComponentList: set_property_to_list,
                   np.ndarray: set_property_to_value_field,
                   pd.DataFrame: set_property_to_value_field}

    def __getitem__(self, item: type):
        bases = [item, *inspect.getmro(type(item))]

        if SimultanObject in bases or MetaMock in bases:
            return set_property_to_sim_component
        else:
            for base in bases:
                if base in self.lookup_dict:
                    return self.lookup_dict[base]
        return None

    def get(self,
            item: Any,
            default='_____'):

        val = self.__getitem__(type(item))
        if val is None:
            if default == '_____':
                raise KeyError(f'No setter function found for type {type(item)}')
            return default
        else:
            return val


type_setter_fcn_lookup_dict = TypeSetterFcnLookupDict()



import typing, abc
from SIMULTAN.Projects import ExtendedProjectData, HierarchicProjectMetaData
from System.IO import FileInfo
from System import MulticastDelegate, IAsyncResult, AsyncCallback
from System.Reflection import MethodInfo
from System.Collections.Generic import List_1, IEnumerable_1

class ManagedComponentFile(ManagedFile):
    def __init__(self, projectData: ExtendedProjectData, _owner: ManagedFileCollection, _file: FileInfo) -> None: ...
    @property
    def CorrespondingResourceIndex(self) -> int: ...
    @CorrespondingResourceIndex.setter
    def CorrespondingResourceIndex(self, value: int) -> int: ...
    @property
    def File(self) -> FileInfo: ...
    @File.setter
    def File(self, value: FileInfo) -> FileInfo: ...
    @property
    def IsUpToDate(self) -> bool: ...
    @property
    def ProjectData(self) -> ExtendedProjectData: ...
    @property
    def PublicCounterpart(self) -> ManagedPublicComponentFile: ...
    @PublicCounterpart.setter
    def PublicCounterpart(self, value: ManagedPublicComponentFile) -> ManagedPublicComponentFile: ...
    def Open(self, _clear_before_open: bool) -> None: ...
    def Save(self) -> None: ...


class ManagedExcelToolFile(ManagedFile):
    def __init__(self, projectData: ExtendedProjectData, _owner: ManagedFileCollection, _file: FileInfo) -> None: ...
    @property
    def CorrespondingResourceIndex(self) -> int: ...
    @CorrespondingResourceIndex.setter
    def CorrespondingResourceIndex(self, value: int) -> int: ...
    @property
    def File(self) -> FileInfo: ...
    @File.setter
    def File(self, value: FileInfo) -> FileInfo: ...
    @property
    def IsUpToDate(self) -> bool: ...
    @property
    def ProjectData(self) -> ExtendedProjectData: ...
    def Open(self, _clear_before_open: bool) -> None: ...
    def Save(self) -> None: ...


class ManagedFile(abc.ABC):
    @property
    def CorrespondingResourceIndex(self) -> int: ...
    @CorrespondingResourceIndex.setter
    def CorrespondingResourceIndex(self, value: int) -> int: ...
    @property
    def File(self) -> FileInfo: ...
    @File.setter
    def File(self, value: FileInfo) -> FileInfo: ...
    @property
    def IsUpToDate(self) -> bool: ...
    @IsUpToDate.setter
    def IsUpToDate(self, value: bool) -> bool: ...
    @property
    def ProjectData(self) -> ExtendedProjectData: ...
    def IsValid(self) -> bool: ...
    def OnDeleted(self, _resource_id: int) -> None: ...
    def OnFileUpToDateChanged(self, isUpToDate: bool) -> None: ...
    def OnRenamed(self, newFile: FileInfo) -> None: ...
    def Open(self, _clear_before_open: bool) -> None: ...
    def Reset(self) -> None: ...
    def Save(self) -> None: ...

    class FileUpToDateChangedEventHandler(MulticastDelegate):
        def __init__(self, object: typing.Any, method: int) -> None: ...
        @property
        def Method(self) -> MethodInfo: ...
        @property
        def Target(self) -> typing.Any: ...
        def BeginInvoke(self, sender: typing.Any, isUpToDate: bool, callback: AsyncCallback, object: typing.Any) -> IAsyncResult: ...
        def EndInvoke(self, result: IAsyncResult) -> None: ...
        def Invoke(self, sender: typing.Any, isUpToDate: bool) -> None: ...



class ManagedFileCollection:
    def __init__(self, _files: List_1[FileInfo], _project_data_manager: ExtendedProjectData) -> None: ...
    @property
    def ComponentEntries(self) -> IEnumerable_1[ManagedComponentFile]: ...
    @ComponentEntries.setter
    def ComponentEntries(self, value: IEnumerable_1[ManagedComponentFile]) -> IEnumerable_1[ManagedComponentFile]: ...
    @property
    def ComponentEntry(self) -> ManagedComponentFile: ...
    @ComponentEntry.setter
    def ComponentEntry(self, value: ManagedComponentFile) -> ManagedComponentFile: ...
    @property
    def ExcelToolEntries(self) -> IEnumerable_1[ManagedExcelToolFile]: ...
    @ExcelToolEntries.setter
    def ExcelToolEntries(self, value: IEnumerable_1[ManagedExcelToolFile]) -> IEnumerable_1[ManagedExcelToolFile]: ...
    @property
    def ExcelToolEntry(self) -> ManagedExcelToolFile: ...
    @ExcelToolEntry.setter
    def ExcelToolEntry(self, value: ManagedExcelToolFile) -> ManagedExcelToolFile: ...
    @property
    def Files(self) -> List_1[ManagedFile]: ...
    @property
    def GeoMapEntries(self) -> IEnumerable_1[ManagedGeoMapFile]: ...
    @GeoMapEntries.setter
    def GeoMapEntries(self, value: IEnumerable_1[ManagedGeoMapFile]) -> IEnumerable_1[ManagedGeoMapFile]: ...
    @property
    def GeometryEntries(self) -> IEnumerable_1[ManagedGeometryFile]: ...
    @GeometryEntries.setter
    def GeometryEntries(self, value: IEnumerable_1[ManagedGeometryFile]) -> IEnumerable_1[ManagedGeometryFile]: ...
    @property
    def GeometryEntry(self) -> ManagedGeometryFile: ...
    @GeometryEntry.setter
    def GeometryEntry(self, value: ManagedGeometryFile) -> ManagedGeometryFile: ...
    @property
    def GeometryRelationsEntries(self) -> IEnumerable_1[ManagedGeometryRelationsFile]: ...
    @GeometryRelationsEntries.setter
    def GeometryRelationsEntries(self, value: IEnumerable_1[ManagedGeometryRelationsFile]) -> IEnumerable_1[ManagedGeometryRelationsFile]: ...
    @property
    def GeometryRelationsEntry(self) -> ManagedGeometryRelationsFile: ...
    @GeometryRelationsEntry.setter
    def GeometryRelationsEntry(self, value: ManagedGeometryRelationsFile) -> ManagedGeometryRelationsFile: ...
    @property
    def LinksFileEntry(self) -> ManagedLinksFile: ...
    @LinksFileEntry.setter
    def LinksFileEntry(self, value: ManagedLinksFile) -> ManagedLinksFile: ...
    @property
    def MetaDataEntry(self) -> ManagedMetaData: ...
    @MetaDataEntry.setter
    def MetaDataEntry(self, value: ManagedMetaData) -> ManagedMetaData: ...
    @property
    def ParameterLibraryEntries(self) -> IEnumerable_1[ManagedParameterFile]: ...
    @ParameterLibraryEntries.setter
    def ParameterLibraryEntries(self, value: IEnumerable_1[ManagedParameterFile]) -> IEnumerable_1[ManagedParameterFile]: ...
    @property
    def ParameterLibraryEntry(self) -> ManagedParameterFile: ...
    @ParameterLibraryEntry.setter
    def ParameterLibraryEntry(self, value: ManagedParameterFile) -> ManagedParameterFile: ...
    @property
    def PublicComponentsEntry(self) -> ManagedPublicComponentFile: ...
    @PublicComponentsEntry.setter
    def PublicComponentsEntry(self, value: ManagedPublicComponentFile) -> ManagedPublicComponentFile: ...
    @property
    def PublicValuesEntry(self) -> ManagedPublicValueFile: ...
    @PublicValuesEntry.setter
    def PublicValuesEntry(self, value: ManagedPublicValueFile) -> ManagedPublicValueFile: ...
    @property
    def SitePlannerEntries(self) -> IEnumerable_1[ManagedSitePlannerFile]: ...
    @SitePlannerEntries.setter
    def SitePlannerEntries(self, value: IEnumerable_1[ManagedSitePlannerFile]) -> IEnumerable_1[ManagedSitePlannerFile]: ...
    @property
    def TaxonomyEntries(self) -> IEnumerable_1[ManagedTaxonomyFile]: ...
    @TaxonomyEntries.setter
    def TaxonomyEntries(self, value: IEnumerable_1[ManagedTaxonomyFile]) -> IEnumerable_1[ManagedTaxonomyFile]: ...
    @property
    def TaxonomyEntry(self) -> ManagedTaxonomyFile: ...
    @TaxonomyEntry.setter
    def TaxonomyEntry(self, value: ManagedTaxonomyFile) -> ManagedTaxonomyFile: ...
    @property
    def UserFileEntry(self) -> ManagedUserFile: ...
    @UserFileEntry.setter
    def UserFileEntry(self, value: ManagedUserFile) -> ManagedUserFile: ...
    @property
    def ValueEntries(self) -> IEnumerable_1[ManagedValueFile]: ...
    @ValueEntries.setter
    def ValueEntries(self, value: IEnumerable_1[ManagedValueFile]) -> IEnumerable_1[ManagedValueFile]: ...
    @property
    def ValueEntry(self) -> ManagedValueFile: ...
    @ValueEntry.setter
    def ValueEntry(self, value: ManagedValueFile) -> ManagedValueFile: ...
    def AddFile(self, _file: FileInfo, projectDataManager: ExtendedProjectData, _load_data: bool = ...) -> None: ...
    def FulfillsMinimumForOpenProject(self) -> bool: ...
    def GetFileWithResourceIndex(self, _index: int) -> ManagedFile: ...
    def HasValidEntries(self) -> bool: ...
    def IsInLoadedState(self) -> bool: ...
    def IsInPostAuthenticationState(self) -> bool: ...
    def IsInPreAuthenticationState(self) -> bool: ...
    def OnManagedFileUpToDateStateChanged(self, files: IEnumerable_1[ManagedFile], all_upToDate: bool) -> None: ...
    def RemoveFile(self, _extension: str) -> None: ...
    def SyncWithResources(self) -> None: ...

    class ManagedFileUpToDateStateChangedEventHandler(MulticastDelegate):
        def __init__(self, object: typing.Any, method: int) -> None: ...
        @property
        def Method(self) -> MethodInfo: ...
        @property
        def Target(self) -> typing.Any: ...
        def BeginInvoke(self, sender: typing.Any, files: IEnumerable_1[ManagedFile], all_upToDate: bool, callback: AsyncCallback, object: typing.Any) -> IAsyncResult: ...
        def EndInvoke(self, result: IAsyncResult) -> None: ...
        def Invoke(self, sender: typing.Any, files: IEnumerable_1[ManagedFile], all_upToDate: bool) -> None: ...



class ManagedGeoMapFile(ManagedFile):
    def __init__(self, projectData: ExtendedProjectData, _owner: ManagedFileCollection, _file: FileInfo) -> None: ...
    @property
    def CorrespondingResourceIndex(self) -> int: ...
    @CorrespondingResourceIndex.setter
    def CorrespondingResourceIndex(self, value: int) -> int: ...
    @property
    def File(self) -> FileInfo: ...
    @File.setter
    def File(self, value: FileInfo) -> FileInfo: ...
    @property
    def IsUpToDate(self) -> bool: ...
    @property
    def ProjectData(self) -> ExtendedProjectData: ...
    def IsValid(self) -> bool: ...
    def Open(self, _clear_before_open: bool) -> None: ...
    def Save(self) -> None: ...


class ManagedGeometryFile(ManagedFile):
    def __init__(self, projectData: ExtendedProjectData, _owner: ManagedFileCollection, _file: FileInfo) -> None: ...
    @property
    def CorrespondingResourceIndex(self) -> int: ...
    @CorrespondingResourceIndex.setter
    def CorrespondingResourceIndex(self, value: int) -> int: ...
    @property
    def File(self) -> FileInfo: ...
    @File.setter
    def File(self, value: FileInfo) -> FileInfo: ...
    @property
    def IsUpToDate(self) -> bool: ...
    @property
    def ProjectData(self) -> ExtendedProjectData: ...
    def IsValid(self) -> bool: ...
    def OnDeleted(self, _resource_id: int) -> None: ...
    def Save(self) -> None: ...


class ManagedGeometryRelationsFile(ManagedFile):
    @typing.overload
    def __init__(self, _original: ManagedFile, _new_file_location: FileInfo) -> None: ...
    @typing.overload
    def __init__(self, projectData: ExtendedProjectData, _owner: ManagedFileCollection, _file: FileInfo) -> None: ...
    @property
    def CorrespondingResourceIndex(self) -> int: ...
    @CorrespondingResourceIndex.setter
    def CorrespondingResourceIndex(self, value: int) -> int: ...
    @property
    def File(self) -> FileInfo: ...
    @File.setter
    def File(self, value: FileInfo) -> FileInfo: ...
    @property
    def IsUpToDate(self) -> bool: ...
    @property
    def ProjectData(self) -> ExtendedProjectData: ...
    def Open(self, _clear_before_open: bool) -> None: ...
    def Save(self) -> None: ...


class ManagedLinksFile(ManagedFile):
    def __init__(self, projectData: ExtendedProjectData, _owner: ManagedFileCollection, _file: FileInfo) -> None: ...
    @property
    def CorrespondingResourceIndex(self) -> int: ...
    @CorrespondingResourceIndex.setter
    def CorrespondingResourceIndex(self, value: int) -> int: ...
    @property
    def File(self) -> FileInfo: ...
    @File.setter
    def File(self, value: FileInfo) -> FileInfo: ...
    @property
    def IsUpToDate(self) -> bool: ...
    @property
    def ProjectData(self) -> ExtendedProjectData: ...
    def Open(self, _clear_before_open: bool) -> None: ...
    def Reset(self) -> None: ...
    def Save(self) -> None: ...


class ManagedMetaData(ManagedFile):
    def __init__(self, projectData: ExtendedProjectData, _owner: ManagedFileCollection, _file: FileInfo) -> None: ...
    @property
    def CorrespondingResourceIndex(self) -> int: ...
    @CorrespondingResourceIndex.setter
    def CorrespondingResourceIndex(self, value: int) -> int: ...
    @property
    def Data(self) -> HierarchicProjectMetaData: ...
    @property
    def File(self) -> FileInfo: ...
    @File.setter
    def File(self, value: FileInfo) -> FileInfo: ...
    @property
    def IsUpToDate(self) -> bool: ...
    @property
    def ProjectData(self) -> ExtendedProjectData: ...
    def Close(self) -> None: ...
    def Open(self, _clear_before_open: bool) -> None: ...
    def Save(self) -> None: ...


class ManagedParameterFile(ManagedFile):
    def __init__(self, projectData: ExtendedProjectData, _owner: ManagedFileCollection, _file: FileInfo) -> None: ...
    @property
    def CorrespondingResourceIndex(self) -> int: ...
    @CorrespondingResourceIndex.setter
    def CorrespondingResourceIndex(self, value: int) -> int: ...
    @property
    def File(self) -> FileInfo: ...
    @File.setter
    def File(self, value: FileInfo) -> FileInfo: ...
    @property
    def IsUpToDate(self) -> bool: ...
    @property
    def ProjectData(self) -> ExtendedProjectData: ...
    def Open(self, _clear_before_open: bool) -> None: ...
    def Save(self) -> None: ...


class ManagedPublicComponentFile(ManagedFile):
    def __init__(self, projectData: ExtendedProjectData, _owner: ManagedFileCollection, _file: FileInfo) -> None: ...
    @property
    def CorrespondingResourceIndex(self) -> int: ...
    @CorrespondingResourceIndex.setter
    def CorrespondingResourceIndex(self, value: int) -> int: ...
    @property
    def File(self) -> FileInfo: ...
    @File.setter
    def File(self, value: FileInfo) -> FileInfo: ...
    @property
    def IsUpToDate(self) -> bool: ...
    @property
    def ProjectData(self) -> ExtendedProjectData: ...
    def Open(self, _clear_before_open: bool) -> None: ...
    def Save(self) -> None: ...


class ManagedPublicValueFile(ManagedFile):
    def __init__(self, projectData: ExtendedProjectData, _owner: ManagedFileCollection, _file: FileInfo) -> None: ...
    @property
    def CorrespondingResourceIndex(self) -> int: ...
    @CorrespondingResourceIndex.setter
    def CorrespondingResourceIndex(self, value: int) -> int: ...
    @property
    def File(self) -> FileInfo: ...
    @File.setter
    def File(self, value: FileInfo) -> FileInfo: ...
    @property
    def IsUpToDate(self) -> bool: ...
    @property
    def ProjectData(self) -> ExtendedProjectData: ...
    def Open(self, _clear_before_open: bool) -> None: ...
    def Save(self) -> None: ...


class ManagedSitePlannerFile(ManagedFile):
    def __init__(self, projectData: ExtendedProjectData, _owner: ManagedFileCollection, _file: FileInfo) -> None: ...
    @property
    def CorrespondingResourceIndex(self) -> int: ...
    @CorrespondingResourceIndex.setter
    def CorrespondingResourceIndex(self, value: int) -> int: ...
    @property
    def File(self) -> FileInfo: ...
    @File.setter
    def File(self, value: FileInfo) -> FileInfo: ...
    @property
    def IsUpToDate(self) -> bool: ...
    @property
    def ProjectData(self) -> ExtendedProjectData: ...
    def IsValid(self) -> bool: ...
    def Open(self, _clear_before_open: bool) -> None: ...
    def Save(self) -> None: ...


class ManagedTaxonomyFile(ManagedFile):
    @typing.overload
    def __init__(self, _original: ManagedFile, _new_file_location: FileInfo) -> None: ...
    @typing.overload
    def __init__(self, projectData: ExtendedProjectData, _owner: ManagedFileCollection, _file: FileInfo) -> None: ...
    @property
    def CorrespondingResourceIndex(self) -> int: ...
    @CorrespondingResourceIndex.setter
    def CorrespondingResourceIndex(self, value: int) -> int: ...
    @property
    def File(self) -> FileInfo: ...
    @File.setter
    def File(self, value: FileInfo) -> FileInfo: ...
    @property
    def IsUpToDate(self) -> bool: ...
    @property
    def LoadedFileVersion(self) -> int: ...
    @LoadedFileVersion.setter
    def LoadedFileVersion(self, value: int) -> int: ...
    @property
    def ProjectData(self) -> ExtendedProjectData: ...
    def Open(self, _clear_before_open: bool) -> None: ...
    def Save(self) -> None: ...


class ManagedUserFile(ManagedFile):
    def __init__(self, projectData: ExtendedProjectData, _owner: ManagedFileCollection, _file: FileInfo) -> None: ...
    @property
    def CorrespondingResourceIndex(self) -> int: ...
    @CorrespondingResourceIndex.setter
    def CorrespondingResourceIndex(self, value: int) -> int: ...
    @property
    def File(self) -> FileInfo: ...
    @File.setter
    def File(self, value: FileInfo) -> FileInfo: ...
    @property
    def IsUpToDate(self) -> bool: ...
    @property
    def ProjectData(self) -> ExtendedProjectData: ...
    def Open(self, _clear_before_open: bool) -> None: ...
    def Save(self) -> None: ...


class ManagedValueFile(ManagedFile):
    def __init__(self, projectData: ExtendedProjectData, _owner: ManagedFileCollection, _file: FileInfo) -> None: ...
    @property
    def CorrespondingResourceIndex(self) -> int: ...
    @CorrespondingResourceIndex.setter
    def CorrespondingResourceIndex(self, value: int) -> int: ...
    @property
    def File(self) -> FileInfo: ...
    @File.setter
    def File(self, value: FileInfo) -> FileInfo: ...
    @property
    def IsUpToDate(self) -> bool: ...
    @property
    def ProjectData(self) -> ExtendedProjectData: ...
    @property
    def PublicCounterpart(self) -> ManagedPublicValueFile: ...
    @PublicCounterpart.setter
    def PublicCounterpart(self, value: ManagedPublicValueFile) -> ManagedPublicValueFile: ...
    @property
    def PublicDependence(self) -> ManagedPublicComponentFile: ...
    @PublicDependence.setter
    def PublicDependence(self, value: ManagedPublicComponentFile) -> ManagedPublicComponentFile: ...
    def Open(self, _clear_before_open: bool) -> None: ...
    def Save(self) -> None: ...




import typing, abc
from System import Attribute, Exception, ValueTuple_2, ValueTuple_3, Array_1
from System.Collections import IDictionary
from System.Reflection import MethodBase
from System.Collections.Generic import IList_1

class BulkElevationProviderAttribute(Attribute):
    def __init__(self) -> None: ...
    @property
    def TypeId(self) -> typing.Any: ...


class ElevationNotFoundException(Exception):
    @typing.overload
    def __init__(self) -> None: ...
    @typing.overload
    def __init__(self, message: str) -> None: ...
    @typing.overload
    def __init__(self, message: str, inner: Exception) -> None: ...
    @property
    def Data(self) -> IDictionary: ...
    @property
    def HelpLink(self) -> str: ...
    @HelpLink.setter
    def HelpLink(self, value: str) -> str: ...
    @property
    def HResult(self) -> int: ...
    @HResult.setter
    def HResult(self, value: int) -> int: ...
    @property
    def InnerException(self) -> Exception: ...
    @property
    def Message(self) -> str: ...
    @property
    def Source(self) -> str: ...
    @Source.setter
    def Source(self, value: str) -> str: ...
    @property
    def StackTrace(self) -> str: ...
    @property
    def TargetSite(self) -> MethodBase: ...


class ElevationProviderAttribute(Attribute):
    def __init__(self) -> None: ...
    @property
    def TypeId(self) -> typing.Any: ...


class IBulkElevationProvider(IElevationProvider, typing.Protocol):
    @abc.abstractmethod
    def GetElevationAtPoints(self, coordinates: IList_1[ValueTuple_2[float, float]], zoomLevel: int = ...) -> IList_1[float]: ...
    @abc.abstractmethod
    def GetTileData(self, pointInTile: ValueTuple_2[float, float], zoomLevel: int = ...) -> ValueTuple_3[Array_1[float], int, int]: ...


class IElevationProvider(typing.Protocol):
    @property
    def MaxZoomLevel(self) -> int: ...
    @abc.abstractmethod
    def GetDisplayName(self) -> str: ...
    @abc.abstractmethod
    def GetElevationAtPoint(self, longitude: float, latitude: float, zoomLevel: int = ...) -> float: ...
    @abc.abstractmethod
    def GetZoomLevelForGridCellSize(self, gridCellSize: float) -> int: ...




from System.IO import FileInfo
from SIMULTAN.Data.MultiValues import SimMultiValueCollection

class ExcelStandardImporter:
    def __init__(self) -> None: ...
    COL_OFFSET : int
    COMPONENT_COL_OFFSET : int
    COMPONENT_NR_VALUE_COLUMNS : int
    MAX_NR_TABLE_ENTRIES : int
    MAX_NR_VALUE_COLUMNS : int
    ROW_OFFSET : int
    TABLE_NAME : str
    def ImportBigTableFromFile(self, file: FileInfo, factory: SimMultiValueCollection, unitX: str, unitY: str, rowNameFormat: str, rowUnit: str, maxRowCount: int = ...) -> None: ...
    def ImportBigTableWNamesFromFile(self, file: FileInfo, factory: SimMultiValueCollection, unitX: str, unitY: str, rowUnit: str, maxRowCount: int = ...) -> None: ...




import typing, abc
from System import Exception, ValueTuple_3
from System.Collections import IDictionary
from System.Reflection import MethodBase
from System.IO import FileInfo
from System.Collections.Generic import List_1

class AccessDeniedException(Exception):
    @typing.overload
    def __init__(self) -> None: ...
    @typing.overload
    def __init__(self, message: str) -> None: ...
    @property
    def Data(self) -> IDictionary: ...
    @property
    def HelpLink(self) -> str: ...
    @HelpLink.setter
    def HelpLink(self, value: str) -> str: ...
    @property
    def HResult(self) -> int: ...
    @HResult.setter
    def HResult(self, value: int) -> int: ...
    @property
    def InnerException(self) -> Exception: ...
    @property
    def Message(self) -> str: ...
    @property
    def Source(self) -> str: ...
    @Source.setter
    def Source(self, value: str) -> str: ...
    @property
    def StackTrace(self) -> str: ...
    @property
    def TargetSite(self) -> MethodBase: ...


class ExceptionToFileWriter(abc.ABC):
    # Skipped Write due to it being static, abstract and generic.

    Write : Write_MethodGroup
    class Write_MethodGroup:
        @typing.overload
        def __call__(self, e: Exception, message: str = ...) -> FileInfo:...
        @typing.overload
        def __call__(self, e: Exception, exceptionFile: FileInfo, message: str = ...) -> FileInfo:...



class FileInUseException(Exception):
    def __init__(self, file: FileInfo) -> None: ...
    @property
    def Data(self) -> IDictionary: ...
    @property
    def File(self) -> FileInfo: ...
    @File.setter
    def File(self, value: FileInfo) -> FileInfo: ...
    @property
    def HelpLink(self) -> str: ...
    @HelpLink.setter
    def HelpLink(self, value: str) -> str: ...
    @property
    def HResult(self) -> int: ...
    @HResult.setter
    def HResult(self, value: int) -> int: ...
    @property
    def InnerException(self) -> Exception: ...
    @property
    def Message(self) -> str: ...
    @property
    def Source(self) -> str: ...
    @Source.setter
    def Source(self, value: str) -> str: ...
    @property
    def StackTrace(self) -> str: ...
    @property
    def TargetSite(self) -> MethodBase: ...


class InvalidGeoReferencingException(Exception):
    def __init__(self) -> None: ...
    @property
    def Data(self) -> IDictionary: ...
    @property
    def HelpLink(self) -> str: ...
    @HelpLink.setter
    def HelpLink(self, value: str) -> str: ...
    @property
    def HResult(self) -> int: ...
    @HResult.setter
    def HResult(self, value: int) -> int: ...
    @property
    def InnerException(self) -> Exception: ...
    @property
    def Message(self) -> str: ...
    @property
    def Source(self) -> str: ...
    @Source.setter
    def Source(self, value: str) -> str: ...
    @property
    def StackTrace(self) -> str: ...
    @property
    def TargetSite(self) -> MethodBase: ...


class InvalidStateException(Exception):
    @typing.overload
    def __init__(self, message: str) -> None: ...
    @typing.overload
    def __init__(self, message: str, innerException: Exception) -> None: ...
    @property
    def Data(self) -> IDictionary: ...
    @property
    def HelpLink(self) -> str: ...
    @HelpLink.setter
    def HelpLink(self, value: str) -> str: ...
    @property
    def HResult(self) -> int: ...
    @HResult.setter
    def HResult(self, value: int) -> int: ...
    @property
    def InnerException(self) -> Exception: ...
    @property
    def Message(self) -> str: ...
    @property
    def Source(self) -> str: ...
    @Source.setter
    def Source(self, value: str) -> str: ...
    @property
    def StackTrace(self) -> str: ...
    @property
    def TargetSite(self) -> MethodBase: ...


class ParameterNotFoundException(Exception):
    def __init__(self, id: int) -> None: ...
    @property
    def Data(self) -> IDictionary: ...
    @property
    def HelpLink(self) -> str: ...
    @HelpLink.setter
    def HelpLink(self, value: str) -> str: ...
    @property
    def HResult(self) -> int: ...
    @HResult.setter
    def HResult(self, value: int) -> int: ...
    @property
    def InnerException(self) -> Exception: ...
    @property
    def Message(self) -> str: ...
    @property
    def ParameterID(self) -> int: ...
    @property
    def Source(self) -> str: ...
    @Source.setter
    def Source(self, value: str) -> str: ...
    @property
    def StackTrace(self) -> str: ...
    @property
    def TargetSite(self) -> MethodBase: ...


class ProjectErrorCode(typing.SupportsInt):
    @typing.overload
    def __init__(self, value : int) -> None: ...
    @typing.overload
    def __init__(self, value : int, force_if_true: bool) -> None: ...
    def __int__(self) -> int: ...
    
    # Values:
    ERR_ON_FILE_DELETE : ProjectErrorCode # 0
    ERR_ON_FILE_DELETE_FILEINUSE : ProjectErrorCode # 1
    ERR_ON_FILE_DELETE_GENERIC : ProjectErrorCode # 2
    ERR_ON_FILE_DELETE_FILENOTFOUND : ProjectErrorCode # 3
    ERR_INVALID_STATE : ProjectErrorCode # 4
    ERR_AUTHSERVICE_NOT_FOUND : ProjectErrorCode # 5


class ProjectFileDeleteException(ProjectIOException):
    def __init__(self, files: List_1[ValueTuple_3[str, ProjectErrorCode, typing.Any]]) -> None: ...
    @property
    def Code(self) -> ProjectErrorCode: ...
    @property
    def Data(self) -> IDictionary: ...
    @property
    def Files(self) -> List_1[ValueTuple_3[str, ProjectErrorCode, typing.Any]]: ...
    @property
    def HelpLink(self) -> str: ...
    @HelpLink.setter
    def HelpLink(self, value: str) -> str: ...
    @property
    def HResult(self) -> int: ...
    @HResult.setter
    def HResult(self, value: int) -> int: ...
    @property
    def InnerException(self) -> Exception: ...
    @property
    def Message(self) -> str: ...
    @property
    def Source(self) -> str: ...
    @Source.setter
    def Source(self, value: str) -> str: ...
    @property
    def StackTrace(self) -> str: ...
    @property
    def TargetSite(self) -> MethodBase: ...


class ProjectIOException(Exception):
    @typing.overload
    def __init__(self, errorCode: ProjectErrorCode, message: str) -> None: ...
    @typing.overload
    def __init__(self, errorCode: ProjectErrorCode, message: str, innerException: Exception) -> None: ...
    @property
    def Code(self) -> ProjectErrorCode: ...
    @property
    def Data(self) -> IDictionary: ...
    @property
    def HelpLink(self) -> str: ...
    @HelpLink.setter
    def HelpLink(self, value: str) -> str: ...
    @property
    def HResult(self) -> int: ...
    @HResult.setter
    def HResult(self, value: int) -> int: ...
    @property
    def InnerException(self) -> Exception: ...
    @property
    def Message(self) -> str: ...
    @property
    def Source(self) -> str: ...
    @Source.setter
    def Source(self, value: str) -> str: ...
    @property
    def StackTrace(self) -> str: ...
    @property
    def TargetSite(self) -> MethodBase: ...


class PropertyUnsupportedValueException(Exception):
    @typing.overload
    def __init__(self) -> None: ...
    @typing.overload
    def __init__(self, message: str) -> None: ...
    @typing.overload
    def __init__(self, message: str, innerException: Exception) -> None: ...
    @property
    def Data(self) -> IDictionary: ...
    @property
    def HelpLink(self) -> str: ...
    @HelpLink.setter
    def HelpLink(self, value: str) -> str: ...
    @property
    def HResult(self) -> int: ...
    @HResult.setter
    def HResult(self, value: int) -> int: ...
    @property
    def InnerException(self) -> Exception: ...
    @property
    def Message(self) -> str: ...
    @property
    def Source(self) -> str: ...
    @Source.setter
    def Source(self, value: str) -> str: ...
    @property
    def StackTrace(self) -> str: ...
    @property
    def TargetSite(self) -> MethodBase: ...

